--------------------------------------------------------
--  DDL for Function GET_IDAPLICACIO
--------------------------------------------------------
spool CARREGAR_FUNCIONS_ABM.log

  CREATE OR REPLACE FUNCTION "APPS"."GET_IDAPLICACIO" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: GET_IDAPLICACIO.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         GET_IDAPLICACIO
   DESCRIPCI:          OBTENIM EL CODI DE L'APLICACIO DONAT UN DETERMINAT PRODUCTE 
   PARAMETRES ENTRADA:  f_idProducte
   PARAMETRES SORTIDA:  v_idAplicacio
   RETORNA:             El codi de l'aplicaci del producte sol.licitat
                       'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI PRODUCTE NO POT SER NULL
                        ERROR: EL PRODUCTE NO EXISTEIX
                        ERROR: EL PARAMETRE ENTRADA NO POT SER NULL                       
   EXEMPLE DE CRIDA:    GET_IDAPLICACIO(254,RSP);
***********************************************************************************/

                        f_idProducte IN PRODUCTE.CODIPRODUCTE%TYPE)
RETURN NUMBER
AS 
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               LOGS.PARAMOUT%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_idAplicacio       number;
    v_OK                number;    

    EX_PRODUCTE_NUL     EXCEPTION;   -- el producte no existeix    
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN
     v_prodediment :=   'GET_IDAPLICACIO ';	
     v_param_entrada := 'CODI PRODUCTE: '||f_idProducte;

-- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA NO SIGUI NULL
          IF f_idProducte IS NULL THEN
              RAISE EX_INS_NUL;
          END IF; 

-- VERIFICA QUE EL PRODUCTE EXISTEIX 
        SELECT COUNT (1)  INTO v_OK
              FROM PRODUCTE
                  WHERE CODIPRODUCTE = f_idproducte;
        IF (v_OK = 0) THEN
            RAISE EX_PRODUCTE_NUL;
        END IF;      

-- CERQUEM EL CODI DE L'APLICACIO CORRESPONENT AL PRODUCTE: f_idProducte

    SELECT IDAPLIC INTO v_idAplicacio 
        FROM PRODUCTE 
            WHERE CODIPRODUCTE = f_idProducte;

-- FINALITZA AMB EXIT
       	v_rsp := 'OK'; 
        DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||v_rsp);
            
-- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);        

COMMIT;

RETURN (v_idAplicacio);

EXCEPTION   

    WHEN EX_PRODUCTE_NUL THEN  -- CONTROL PRODUCTE EXISTEIX
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: AQUEST PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_idAplicacio:= -1;
          RETURN (v_idAplicacio);
          
    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI PRODUCTE NO POT SER NULL '||v_error|| ': '||v_desc;
             
       -- INSERTEM L'ERROR A LA TAULA DE LOGS
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_idAplicacio:= -1;
          RETURN (v_idAplicacio);        

    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_idAplicacio:= -1;
          RETURN (v_idAplicacio);
          
END GET_IDAPLICACIO;

/
--------------------------------------------------------
--  DDL for Function GET_IDSSOO
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "APPS"."GET_IDSSOO" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: GET_IDSSOO.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         GET_IDSSOO
   DESCRIPCI:          OBTENIM EL CODI DEL SISTEMA OPERATIU DONAT UN DETERMINAT PRODUCTE 
   PARAMETRES ENTRADA:  f_idProducte
   PARAMETRES SORTIDA:  v_ssoo
   RETORNA:             El codi del sistema operatiu del producte sol.licitat
                       'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI SSOO NO POT SER NULL
                        ERROR: EL PRODUCTE NO EXISTEIX
                        ERROR: EL PARAMETRE ENTRADA NO POT SER NULL     
   EXEMPLE DE CRIDA:    GET_IDSSOO(254,RSP);
***********************************************************************************/

                        f_idProducte IN PRODUCTE.CODIPRODUCTE%TYPE)
RETURN NUMBER
AS 
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               LOGS.PARAMOUT%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_ssoo              number;
    v_OK                number;    

    EX_PRODUCTE_NUL     EXCEPTION;   -- el producte no existeix    
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    
BEGIN
    v_prodediment :=   'GET_SSOO ';	
    v_param_entrada := 'CODI PRODUCTE: '||f_idProducte;

-- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA NO SIGUI NULL
          IF f_idProducte IS NULL THEN
              RAISE EX_INS_NUL;
          END IF; 

-- VERIFICA QUE EL PRODUCTE EXISTEIX 
        SELECT COUNT (1)  INTO v_OK
              FROM PRODUCTE
                  WHERE CODIPRODUCTE = f_idproducte;
        IF (v_OK = 0) THEN
            RAISE EX_PRODUCTE_NUL;
        END IF; 

-- CERQUEM EL CODI DEL SISTEMA OPERATIU DE L'APLICACIO CORRESPONENT AL PRODUCTE: f_idProducte

    SELECT IDSSOO INTO v_ssoo  
        FROM PRODUCTE 
            WHERE CODIPRODUCTE = f_idProducte;

-- FINALITZA AMB EXIT
       	v_rsp := 'OK'; 
        DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||v_rsp);
            
-- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);        

COMMIT;

RETURN (v_ssoo);

EXCEPTION   

    WHEN EX_PRODUCTE_NUL THEN  -- CONTROL PRODUCTE EXISTEIX
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: AQUEST PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_ssoo:= -1;
          RETURN (v_ssoo);
          
    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI SSOO NO POT SER NULL '||v_error|| ': '||v_desc;
             
       -- INSERTEM L'ERROR A LA TAULA DE LOGS
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_ssoo:= -1;
          RETURN (v_ssoo);        

    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_ssoo:= -1;
          RETURN (v_ssoo);    
          
END GET_IDSSOO;

/
--------------------------------------------------------
--  DDL for Function GET_NOM_APLICACIO
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "APPS"."GET_NOM_APLICACIO" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: GET_NOM_APLICACIO.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         GET_NOM_APLICACIO
   DESCRIPCI:          OBTENIM EL NOM COMPLERT DE L'APLICACIO DONAT UN DETERMINAT PRODUCTE 
   PARAMETRES ENTRADA:  f_idProducte
   PARAMETRES SORTIDA:  v_nomAplicacio
   RETORNA:             El nom complert de l'aplicaci del producte sol.licitat, 
                        EXEMPLE DE RETORN: "FLASH PLAYER  ANDROID v.1.1"
                       'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PRODUCTE NO EXISTEIX
                        ERROR: EL PARAMETRE ENTRADA CODI PRODUCTE NO POT SER NULL
   EXEMPLE DE CRIDA:    GET_NOM_APLICACIO(254,RSP);
***********************************************************************************/

                        f_idProducte IN PRODUCTE.CODIPRODUCTE%TYPE)
RETURN VARCHAR2 AS 

    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               LOGS.PARAMOUT%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_nomAplicacio      VARCHAR2(70);
    v_OK                number;    

    EX_PRODUCTE_NUL     EXCEPTION;   -- el producte no existeix    
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN
    v_prodediment :=   'GET_NOM_APLICACIO ';	
    v_param_entrada := 'CODI PRODUCTE: '||f_idProducte;

-- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA NO SIGUI NULL
          IF f_idProducte IS NULL THEN
              RAISE EX_INS_NUL;
          END IF; 

  -- VERIFICA QUE EL PRODUCTE EXISTEIX 
        SELECT COUNT (1)  INTO v_OK
              FROM PRODUCTE
                  WHERE CODIPRODUCTE = f_idproducte;
        IF (v_OK = 0) THEN
            RAISE EX_PRODUCTE_NUL;
        END IF;      
        
   -- CERQUEM EL NOM COMPLERT DE L'APLICACIO: APLICACIO + SSOO + VERSIO
          SELECT AP.NOMAPLI||' '||S.NOMSSOO||' '||P.VERSIO INTO v_nomAplicacio
            FROM APLICACIO AP, PRODUCTE P ,SSOO S
                  WHERE AP.IDAPLICACIO = P.IDAPLIC
                    AND S.IDSSOO         = P.IDSSOO
                    AND P.CODIPRODUCTE = f_idProducte;

   -- FINALITZA AMB EXIT
          	v_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||v_rsp);
            
   -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
            
       COMMIT;
                    
RETURN (v_nomAplicacio);

EXCEPTION   

    WHEN EX_PRODUCTE_NUL THEN  -- CONTROL PRODUCTE EXISTEIX
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: AQUEST PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_nomAplicacio:= '-1';
          RETURN (v_nomAplicacio);
          
    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI PRODUCTE NO POT SER NULL '||v_error|| ': '||v_desc;
             
       -- INSERTEM L'ERROR A LA TAULA DE LOGS
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_nomAplicacio:= '-1';
          RETURN (v_nomAplicacio);               
    
    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_nomAplicacio:= '-1';
          RETURN (v_nomAplicacio);
          
END GET_NOM_APLICACIO;

/
--------------------------------------------------------
--  DDL for Function GET_NOM_DESENV
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "APPS"."GET_NOM_DESENV" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: GET_NOM_DESENV.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         GET_NOM_DESENV
   DESCRIPCI:          OBTENIM EL NOM DEL/S DESENVOLUPADOR/S D'UNA APLICACIO 
                        DONAT UN DETERMINAT PRODUCTE 
   PARAMETRES ENTRADA:  f_idProducte
   PARAMETRES SORTIDA:  v_nomsdesen
   RETORNA:             El nom del/s desenvolupador/s de l'aplicaci del producte 
                        del parametre d'entrada: f_idProducte
                        EXEMPLE DE RETORN: "LEVELUP; WHATSAPP INC; WIKIPEDIA FOUNATION INC; "
                       'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PRODUCTE NO EXISTEIX
                        ERROR: EL PRODUCTE NO TE CAP DESENVOLUPADOR ASSIGNAT
                        ERROR: EL PARAMETRE ENTRADA CODI PRODUCTE NO POT SER NULL                        
   EXEMPLE DE CRIDA:    GET_NOM_DESENV(254,RSP);
***********************************************************************************/
                        f_idProducte IN PRODUCTE.CODIPRODUCTE%TYPE)

RETURN VARCHAR2 AS 

    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               LOGS.PARAMOUT%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_noms              desenvolupador.nomdesen%TYPE;
    v_nomsdesen         VARCHAR2(200);
    v_nomDesenvApl      desenvolupador.nomdesen%TYPE;
    v_OK                number;
        
    EX_DESENV_NUL       EXCEPTION;   -- el producte te cap DESENVOLUPADOR 
    EX_PRODUCTE_NUL     EXCEPTION;   -- el producte no existeix    
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

    CURSOR C_NOM_DES IS   
              SELECT D.NOMDESEN INTO v_nomDesenvApl
                FROM AUTOR AU , DESENVOLUPADOR D, PRODUCTE P
                  WHERE D.IDDESENV = AU.IDDESEV
                    AND P.CODIPRODUCTE = AU.IDPROD
                    AND P.CODIPRODUCTE = f_idProducte ;

BEGIN

    v_prodediment :=   'GET_NOM_DESENV ';	
    v_param_entrada := 'CODI PRODUCTE: '||f_idProducte;

  -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA NO SIGUI NULL
        IF f_idProducte IS NULL THEN
              RAISE EX_INS_NUL;
        END IF; 
          
   -- VERIFICA QUE EL PRODUCTE EXISTEIX 
        SELECT COUNT (1)  INTO v_OK
              FROM PRODUCTE
                  WHERE CODIPRODUCTE = f_idproducte;
                  
        IF (v_OK = 0) THEN
            RAISE EX_PRODUCTE_NUL;
        END IF;      
        
  -- VERIFICA QUE EL PRODUCTE TINGUI ASSIGNAT UN DESENVOLUPADOR
        SELECT COUNT(IDDESEV) INTO v_OK
              FROM AUTOR
                  WHERE IDPROD = f_idproducte;
        IF (v_OK = 0) THEN
            RAISE EX_DESENV_NUL;
        END IF;  
    
  -- APERTURA DEL CURSOR PER CERCAR EL NOM DEL/S DESENVOLUPADOR/S
          OPEN C_NOM_DES;
          
  -- EXTRACCIO DE LES DADES
          LOOP
              FETCH C_NOM_DES INTO
                             v_noms;
              EXIT WHEN C_NOM_DES%NOTFOUND;
                            v_nomsdesen:= v_nomsdesen||v_noms||'; ';
          END LOOP;
                          DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||v_nomsdesen);
   -- TANCAR EL CURSOR   
          CLOSE C_NOM_DES;

   -- FINALITZA AMB EXIT
          v_rsp := 'OK'; 
          DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||v_rsp);
            
   -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
COMMIT;
       
RETURN (v_nomsdesen);

EXCEPTION   

    WHEN EX_PRODUCTE_NUL THEN  -- CONTROL PRODUCTE EXISTEIX
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: AQUEST PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_nomsdesen:= '-1';
          RETURN (v_nomsdesen);
          
    WHEN EX_DESENV_NUL THEN  -- CONTROL: el producte no te cap DESENVOLUPADOR assignat
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: PRODUCTE SENSE DESENVOLUPADOR '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);  
          v_nomsdesen:= '-1';
          RETURN (v_nomsdesen);

    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' EL PARAMETRE ENTRADA CODI PRODUCTE NO POT SER NULL '||v_error|| ': '||v_desc;
             
       -- INSERTEM L'ERROR A LA TAULA DE LOGS
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_nomsdesen:= '-1';
          RETURN (v_nomsdesen);         
          
    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_nomsdesen:= '-1';
          RETURN (v_nomsdesen);
          
END GET_NOM_DESENV;

/
--------------------------------------------------------
--  DDL for Function GET_PREU_APAGAR
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "APPS"."GET_PREU_APAGAR" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: GET_PREU_APAGAR.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         GET_PREU_APAGAR
   DESCRIPCI:          OBTENIM EL PREU A PAGAR DE LA DESCARREGA EN FUNCIO 
                        DE SI L'APLICACIO S'HAVIA DESCARREGAT ABANS EN EL MATEIX DISPOSITIU.
   PARAMETRES ENTRADA:  f_idProducte, f_idDispo, f_idPais
   PARAMETRES SORTIDA:  v_preu
   RETORNA:             L'import a pagar per la descarrega
                       'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PRODUCTE NO EXISTEIX
                        ERROR: EL DISPOSITIU NO EXISTEIX
                        ERROR: EL PAIS NO EXISTEIX
                        ERROR: ELS PARAMETRES ENTRADA: CODI PRODUCTE,DISPOSITIU,PAIS NO PODEN SER NULL                       
   EXEMPLE DE CRIDA:    GET_PREU_APAGAR(254, 54, 1);
***********************************************************************************/
                        f_idProducte IN PRODUCTE.CODIPRODUCTE%TYPE, 
                        f_idDispo    IN DISPOSITIU.IDDISPOSITIU%TYPE,
                        f_idPais     IN USUARIS.IDPAISREGISTRE%TYPE)
RETURN NUMBER
 
AS 

/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               LOGS.PARAMOUT%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_preu              number;
    v_aplic             PRODUCTE.IDAPLIC%TYPE;
    v_ssoo              PRODUCTE.IDSSOO%TYPE;
    v_Descarregues      number;
    v_OK                number;
    
    EX_PAIS_NUL         EXCEPTION;   -- el pais no existeix 
    EX_DISPOSITIU_NUL   EXCEPTION;   -- el dispositiu no existeix 
    EX_PRODUCTE_NUL     EXCEPTION;   -- el producte no existeix    
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN
     v_prodediment :=   'GET_PREU_APAGAR ';	
     v_param_entrada := 'CODI PRODUCTE: '                     ||f_idProducte||
                        ' / CODI DISPOSITU: '                 ||f_idDispo||
                        ' / CODI PAIS REGISTRE USUARI: '      ||f_idPais;
        
-- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (f_idProducte IS NULL OR f_idDispo IS NULL OR f_idPais IS NULL ) THEN
            RAISE EX_INS_NUL;
        END IF; 

-- VERIFIQUEM QUE EL PRODUCTE EXISTEIX 
        SELECT COUNT (1)  INTO v_OK
              FROM PRODUCTE
                  WHERE CODIPRODUCTE = f_idproducte;
        IF (v_OK = 0) THEN
            RAISE EX_PRODUCTE_NUL;
        END IF;     

-- VERIFICA QUE EL DISPOSITIU EXISTEIXI
        SELECT COUNT (1)  INTO v_OK
              FROM DISPOSITIU
                  WHERE IDDISPOSITIU = f_idDispo;
        IF (v_OK = 0) THEN
            RAISE EX_DISPOSITIU_NUL;
        END IF;      
        
-- VERIFIQUEM QUE EL PAIS EXISTEIXI 
        SELECT COUNT (1)  INTO v_OK
              FROM PAIS
                  WHERE IDPAIS = f_idPais;
        IF (v_OK = 0) THEN
            RAISE EX_PAIS_NUL;
        END IF;           
        
-- VERIFIQUEM SI S'HA DESCARREGAT ANTERIORMENT L'APLICACIO EN AQUEST DISPOSITIU

    -- GET v_aplicacio
        v_aplic:= GET_IDAPLICACIO(f_idProducte);
    -- GET v_ssoo
        v_ssoo := GET_IDSSOO(f_idProducte);

        SELECT  COUNT(1) INTO v_Descarregues
            FROM DESCARREGA
                WHERE IDDISPOSITIU = f_idDispo AND IDPRODUCTE IN 
                    (SELECT CODIPRODUCTE FROM PRODUCTE WHERE IDAPLIC= v_aplic AND IDSSOO = v_ssoo);

-- OBTENIM EL PREU DE LA DESCARREGA

        IF (v_Descarregues > 0) THEN
              v_preu := 0;
        ELSE
              SELECT PREUAPLIC INTO v_preu 
                  FROM PREUS
                    WHERE IDPRODUCTE = f_idProducte AND IDPAIS = f_idPais;
        END IF;

-- FINALITZA AMB EXIT
       	v_rsp := 'OK'; 
        DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: PREU A PAGAR: '||TO_CHAR(v_preu,'999,990.99')||' '||v_rsp);
            
-- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||TO_CHAR(v_preu,'999,990.99')||' '||v_rsp);        

COMMIT;

RETURN (v_preu);
  
EXCEPTION   

    WHEN EX_PRODUCTE_NUL THEN  -- CONTROL PRODUCTE EXISTEIXI
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: AQUEST PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_preu:= -1;
          RETURN (v_preu);
      
    WHEN EX_PAIS_NUL THEN  -- CONTROL PAIS EXISTEIXI
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: AQUEST PAIS NO EXISTEIX '||v_error|| ': '||v_desc;

       -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_preu:= -1;
          RETURN (v_preu);

    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR: ELS PARAMETRES ENTRADA: CODI PRODUCTE,DISPOSITIU,PAIS NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
       -- INSERTEM L'ERROR A LA TAULA DE LOGS
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_preu:= -1;
          RETURN (v_preu);           
          
    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
          v_error := SQLCODE;
          v_desc := SUBSTR(SQLERRM, 1, 256);
          v_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
      -- INSERTEM L'ERROR A LA TAULA DE LOG
          ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
          DBMS_OUTPUT.PUT_LINE('RSP: '||v_rsp);
          v_preu:= -1;
          RETURN (v_preu);        
          
END GET_PREU_APAGAR;

/
spool OFF
