--------------------------------------------------------
--  DDL for Package PKG_ABM_APLICACIONS
--------------------------------------------------------

spool CARREGAR_PROCEDIMENTS_ABM.log

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_APLICACIONS" AS 
 
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: PKG_ABM_APLICACIONS.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_APLICACIO
   DESCRIPCI:          AFEGEIX LES DADES D'UNA NOVA APLICACIO A LA TAULA APLICACIO
   PARAMETRES ENTRADA:  a_nomaplicacio, a_idcategoria
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: NOM APLICACIO, NO POT HAVER-HI DUPLICATS
   EXEMPLE DE CRIDA:     PKG_ABM_APLICACIONS.ALTA_APLICACIO ('MAHJONG', 1,RSP);
***********************************************************************************/
 PROCEDURE ALTA_APLICACIO (
                            a_nomaplicacio  IN APLICACIO.NOMAPLI%TYPE,
                            a_idcategoria   IN APLICACIO.IDCATEGORIA%TYPE,
                            a_RSP           OUT NOCOPY LOGS.paramOut%TYPE);
        
/************************************************************************************
   PROCEDIMENT:         MODIF_APLICACIO
   DESCRIPCI:          MODIFICA LES DADES D'UNA APLICACIO EXISTENT A LA TAULA APLICACIO
                        SI UNA APLICACIO CANVIA DE NOM S'ACTUALITZA A L'ESTADISTICA 
                        MODUL_EST_345
   PARAMETRES ENTRADA:  m_idaplicacio, m_nomaplicacio, m_idcategoria
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: EL PARAMETRE ENTRADA CODI APLICACIO NO POT SER NULL
                         ERROR: APLICACIO NO EXISTEIX
                         ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: NOM APLICACIO, NO POT HAVER-HI DUPLICATS                        
   EXEMPLE DE CRIDA:     PKG_ABM_APLICACIONS.MODIF_APLICACIO (21,'MAHJONG', 2, RSP);
***********************************************************************************/  
 PROCEDURE MODIF_APLICACIO (
                            m_idaplicacio   IN APLICACIO.IDAPLICACIO%TYPE,
                            m_nomaplicacio  IN APLICACIO.NOMAPLI%TYPE,
                            m_idcategoria   IN APLICACIO.IDCATEGORIA%TYPE,
                            m_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
        
/************************************************************************************
   PROCEDIMENT:         BAIXA_APLICACIO
   DESCRIPCI:          ESBORRA LES DADES D'UNA APLICACIO EXISTENT SI NO TE PRODUCTES ASSOCIATS
   PARAMETRES ENTRADA:  b_idaplicacio
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI APLICACIO NO POT SER NULL
                        ERROR: APLICACIO NO EXISTEIX
                        ERROR: L'APLICACIO ESTA ASSOCIADA A ALGUN PRODUCTE
   EXEMPLE DE CRIDA:    PKG_ABM_APLICACIONS.BAIXA_APLICACIO (21,RSP);
***********************************************************************************/
PROCEDURE BAIXA_APLICACIO (
                            b_idaplicacio   IN APLICACIO.IDAPLICACIO%TYPE,
                            b_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/* VARIABLES GLOBALS */
    v_prodediment        LOGS.PROCEDIMENT%TYPE;
    v_param_entrada      LOGS.PARAMIN%TYPE; 
    v_id                 APLICACIO.IDAPLICACIO%TYPE;
    v_Total              number;
    v_error              number;
    v_desc               VARCHAR2(256);
    v_NOM_old            APLICACIO.NOMAPLI%TYPE;

    EX_INS_NUL           EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL            EXCEPTION;  -- no existeix la clau forana
    EX_APLICACIO_ACTIU   EXCEPTION;  -- no es pot esborrar l'aplicacio que tingui productes associats
    EX_IDAPLIC_NUL       EXCEPTION;  -- el codi de l'APLICACIO no pot ser null
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
          

END PKG_ABM_APLICACIONS;

/
--------------------------------------------------------
--  DDL for Package PKG_ABM_AUTOR
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_AUTOR" AS 

 /************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: PKG_ABM_AUTOR.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_AUTOR
   DESCRIPCI:          AFEGEIX UNA NOU AUTOR D'UNA APLICACIO EN UN SSOO DETERMINAT I 
                        UNA VERSIO DETERMINADA A LA TAULA AUTOR
                        ACTUALITZA A L'ESTADISTICA MODUL_EST_345
   PARAMETRES ENTRADA:  a_idprod, a_iddesev
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: PRODUCTE + DESENVOLUPADOR, NO POT HAVER-HI DUPLICATS
   EXEMPLE DE CRIDA:    ALTA_AUTOR (254,2,RSP);
***********************************************************************************/
 PROCEDURE ALTA_AUTOR (
                          a_idprod       IN AUTOR.IDPROD%TYPE,
                          a_iddesev      IN AUTOR.IDDESEV%TYPE,
                          a_rsp          OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         MODIF_AUTOR
   DESCRIPCI:          MODIFICA L'AUTOR D'UN D'UNA APLICACIO EN UN SSOO DETERMINAT I 
                        UNA VERSIO DETERMINADA EXISTENT A LA TAULA AUTOR
                        ACTUALITZA A L'ESTADISTICA MODUL_EST_345
   PARAMETRES ENTRADA:  m_idautor, m_idprod, m_iddesev
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI AUTOR NO POT SER NULL
                        ERROR: AUTOR NO EXISTEIX
                        ERROR: CLAU FORANA INCORRECTA
                        ERROR: VALOR NO POT SER NULL
                        ERROR: CLAU UNICA: PRODUCTE + DESENVOLUPADOR, NO POT HAVER-HI DUPLICATS
   EXEMPLE DE CRIDA:    MODIF_AUTOR (421,254,2,RSP);
***********************************************************************************/  
 PROCEDURE MODIF_AUTOR (
                          m_idautor       IN AUTOR.IDAUTOR%TYPE,
                          m_idprod        IN AUTOR.IDPROD%TYPE,
                          m_iddesev       IN AUTOR.IDDESEV%TYPE,
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         BAIXA_AUTOR
   DESCRIPCI:          ESBORRA L'AUTOR D'UN D'UNA APLICACIO EN UN SSOO DETERMINAT I 
                        UNA VERSIO DETERMINADA EXISTENT A LA TAULA AUTOR
                        ACTUALITZA A L'ESTADISTICA MODUL_EST_345
   PARAMETRES ENTRADA:  b_idautor
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI AUTOR NO POT SER NULL
                        ERROR: AUTOR NO EXISTEIX
                        ERROR: AUTOR EN ESTAT INACTIU
   EXEMPLE DE CRIDA:    BAIXA_AUTOR(254,2);
***********************************************************************************/
PROCEDURE BAIXA_AUTOR (
                        b_idautor       IN AUTOR.IDAUTOR%TYPE,
                        b_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/* VARIABLES GLOBALS */
          v_prodediment      LOGS.PROCEDIMENT%TYPE;
          v_param_entrada    LOGS.PARAMIN%TYPE; 
          v_id               AUTOR.IDAUTOR%TYPE;
          v_error            number;
          v_desc             VARCHAR2(256);
          v_Total            number;

          EX_INS_NUL         EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
          EX_FK_NUL          EXCEPTION;  -- no existeix la clau forana
          EX_IDAUTOR_NUL     EXCEPTION;  -- el codi de l'AUTOR no pot ser null          

    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291); 

END PKG_ABM_AUTOR;

/
--------------------------------------------------------
--  DDL for Package PKG_ABM_DESCRIP
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_DESCRIP" AS 
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: ALTA_DESCRIP.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_DESCRIP
   DESCRIPCI:          AFEGEIX UNA NOVA DESCRIPCIO D'UNA APLICACIO EN UN IDIOMA 
                        DETERMINAT A LA TAULA DESCRIPCIO                   
   PARAMETRES ENTRADA:  a_idaplicacio, a_ididioma, a_descrip
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
  ERRORS:                ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA:  APLICACIO + IDIOMA, NO POT HAVER-HI DUPLICATS                        
   EXEMPLE DE CRIDA:     PKG_ABM_DESCRIP.ALTA_DESCRIP (25,2,' XXXXXX',RSP);
***********************************************************************************/
 PROCEDURE ALTA_DESCRIP (
                            a_idaplicacio   IN DESCRIPCIO.IDAPLICACIO%TYPE,
                            a_ididioma      IN DESCRIPCIO.IDIDIOMA%TYPE,
                            a_descrip       IN DESCRIPCIO.DESCRIPCIO%TYPE,
                            a_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         MODIF_DESCRIP
   DESCRIPCI:          MODIFICA LA DESCRIPCIO D'UNA APLICACIO EXISTENT                    
   PARAMETRES ENTRADA:  m_iddescripcio, m_idaplicacio, m_ididioma, m_descrip
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: EL PARAMETRE ENTRADA CODI DESCRIPCIO NO POT SER NULL
                         ERROR: DESCRIPCIO NO EXISTEIX
                         ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: APLICACIO + IDIOMA, NO POT HAVER-HI DUPLICATS                         
   EXEMPLE DE CRIDA:     PKG_ABM_DESCRIP.MODIF_DESCRIP (21,25,2,'   xxx',RSP);
***********************************************************************************/  
 PROCEDURE MODIF_DESCRIP (
                            m_iddescrip     IN DESCRIPCIO.IDDESCRIPCIO%TYPE,
                            m_idaplicacio   IN DESCRIPCIO.IDAPLICACIO%TYPE,
                            m_ididioma      IN DESCRIPCIO.IDIDIOMA%TYPE,
                            m_descrip       IN DESCRIPCIO.DESCRIPCIO%TYPE,
                            m_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         BAIXA_DESCRIP
   DESCRIPCI:          ESBORRA LA DESCRIPCIO D'UNA APLICACIO D'UN IDIOMA DETEMINAT 
                        DE LA TAULA DESCRIPCIO
   PARAMETRES ENTRADA:  b_iddescrip   
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
                         ERROR: EL PARAMETRE ENTRADA CODI DESCRIPCIO NO POT SER NULL               
                         ERROR: DESCRIPCIO NO EXISTEIX
   EXEMPLE DE CRIDA:     PKG_ABM_DESCRIP.BAIXA_DESCRIP(21,RSP);
***********************************************************************************/
PROCEDURE BAIXA_DESCRIP (
                          b_iddescrip     IN DESCRIPCIO.IDDESCRIPCIO%TYPE,
                          b_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/* VARIABLES GLOBALS */
          v_prodediment      LOGS.PROCEDIMENT%TYPE;
          v_param_entrada    LOGS.PARAMIN%TYPE;          
          v_id               DESCRIPCIO.IDDESCRIPCIO%TYPE;
          v_Total            number;
          v_error            number;
          v_desc             VARCHAR2(256);

          EX_INS_NUL         EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
          EX_FK_NUL          EXCEPTION;  -- no existeix la clau forana
          EX_IDDESCR_NUL     EXCEPTION;  -- el codi de la DESCRIPCIO no pot ser null

    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
   

END PKG_ABM_DESCRIP;

/
--------------------------------------------------------
--  DDL for Package PKG_ABM_DISPOSITIU
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_DISPOSITIU" AS 

 /************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: PKG_ABM_DISPOSITIU.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_DISPOSITIU
   DESCRIPCI:          AFEGEIX LES DADES D'UN DISPOSITIU MOBIL A LA TAULA DISPOSITIU
   PARAMETRES ENTRADA:  a_idusuari, a_codiimei,a_idmodel,a_idSSOO, a_versiossoo,
                        a_resolucio, a_ididiomadisp
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: CODI IMEI, NO POT HAVER-HI DUPLICATS
                         ERROR: CODI IMEI HA DE TENIR 15 DIGITS
  EXEMPLE DE CRIDA:      PKG_ABM_DISPOSITIU.ALTA_DISPOSITIU (245,'123456789012345',5,1,'4.1','320 x 480 pixels',1,RSP);
***********************************************************************************/
 PROCEDURE ALTA_DISPOSITIU (
                              a_idusuari      IN DISPOSITIU.IDUSUARI%TYPE,
                              a_codiimei      IN DISPOSITIU.CODIIMEI%TYPE,
                              a_idmodel       IN DISPOSITIU.IDMODEL%TYPE,
                              a_idSSOO        IN DISPOSITIU.IDSSOO%TYPE,
                              a_versiossoo    IN DISPOSITIU.VERSIOSSOO%TYPE,      
                              a_resolucio     IN DISPOSITIU.RESOLUCIO%TYPE,
                              a_ididiomadisp  IN DISPOSITIU.IDIDIOMADISP%TYPE,          
                              a_RSP           OUT NOCOPY LOGS.paramOut%TYPE);
              
/************************************************************************************
   PROCEDIMENT:         MODIF_DISPOSITIU
   DESCRIPCI:          MODIFICA LES DADES D'UN DISPOSITIU EXISTENT A LA TAULA DISPOSITIU
   PARAMETRES ENTRADA:  m_iddispositiu, m_idusuari, m_codiimei, m_idmodel,m_idSSOO, 
                        m_versiossoo, m_resolucio, m_ididiomadisp, m_estat
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: EL PARAMETRE ENTRADA CODI DISPOSITIU NO POT SER NULL
                         ERROR: DISPOSITIU NO EXISTEIX
                         ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: CODI IMEI, NO POT HAVER-HI DUPLICATS                        
                         ERROR: ERROR: ESTAT NOMS POT SER A (ALTA)
                         ERROR: CODI IMEI HA DE TENIR 15 DIGITS
   EXEMPLE DE CRIDA:     PKG_ABM_DISPOSITIU.MODIF_DISPOSITIU (41,245,'123456789012345',5,1,'4.1',
                                          '320 x 480 pixels',1,'A',RSP);
***********************************************************************************/  
 PROCEDURE MODIF_DISPOSITIU (
                              m_iddispositiu    IN DISPOSITIU.IDDISPOSITIU%TYPE,
                              m_idusuari        IN DISPOSITIU.IDUSUARI%TYPE,
                              m_codiimei        IN DISPOSITIU.CODIIMEI%TYPE,
                              m_idmodel         IN DISPOSITIU.IDMODEL%TYPE,
                              m_idSSOO          IN DISPOSITIU.IDSSOO%TYPE,
                              m_versiossoo      IN DISPOSITIU.VERSIOSSOO%TYPE,      
                              m_resolucio       IN DISPOSITIU.RESOLUCIO%TYPE,
                              m_ididiomadisp    IN DISPOSITIU.IDIDIOMADISP%TYPE,  
                              m_estat           IN DISPOSITIU.ESTAT%TYPE, 
                              m_rsp             OUT NOCOPY LOGS.paramOut%TYPE);
              
/************************************************************************************
   PROCEDIMENT:         BAIXA_DISPOSITIU
   DESCRIPCI:          ESBORRA LES DADES D'UN DISPOSITIU EXISTENT:
                        DISPOSITIU SENSE DESCARREGUES ASSOCIADES: ESBORRA DE LA TAULA DISPOSITIUS
                        DISPOSITIU AMB DESCARREGUES ASSOCIADES: CANVIA L'ESTAT A BAIXA 
   PARAMETRES ENTRADA:  b_iddispositiu
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI DISPOSITIU NO POT SER NULL
                        ERROR: DISPOSITIU NO EXISTEIX
                        ERROR: DISPOSITIU EN ESTAT INACTIU                        
   EXEMPLE DE CRIDA:    PKG_ABM_DISPOSITIU.BAIXA_DISPOSITIU (41,RSP);
***********************************************************************************/
PROCEDURE BAIXA_DISPOSITIU (
                              b_iddispositiu   IN DISPOSITIU.IDDISPOSITIU%TYPE,
                              b_rsp            OUT NOCOPY LOGS.paramOut%TYPE);
              
/* VARIABLES GLOBALS */
    v_prodediment LOGS.PROCEDIMENT%TYPE;
    v_param_entrada LOGS.PARAMIN%TYPE; 
    v_id DISPOSITIU.IDDISPOSITIU%TYPE;
    v_error number;
    v_desc VARCHAR2(256);
    v_Total number;
    v_estat             DISPOSITIU.ESTAT%TYPE;

    EX_INS_NUL            EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL             EXCEPTION;   -- no existeix la clau forana
    EX_DATA_NO_OK         EXCEPTION;   -- data no correcta
    EX_EST_NOK            EXCEPTION;   -- l'estat noms pot ser 'A' (ALTA)
    EX_IMEI_NOK           EXCEPTION;   -- el codi IMEI ha de tenir 15 digits
    EX_DISPOSITIU_INACTIU EXCEPTION;   -- si el DISPOSITIU ja esta en estat de baixa
    EX_IDDISP_NUL         EXCEPTION;   -- el codi del DISPOSITIU no pot ser null

    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
 
END PKG_ABM_DISPOSITIU;

/
--------------------------------------------------------
--  DDL for Package PKG_ABM_PREUS
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_PREUS" AS 
 
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: PKG_ABM_PREUS.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_PREUS
   DESCRIPCI:          AFEGEIX UNA NOU PREU D'UN PRODUCTE EN UN PAIS
                        DETERMINAT A LA TAULA PREUS 
                        (LA DIVISA EUROS, S'AFEGEIX AUTOMATICAMENT EN EL TRIGGER: INS_PREUS_idPreus)
   PARAMETRES ENTRADA:  a_idproducte, a_idpais, a_preuaplic
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: PRODUCTE + PAIS, NO POT HAVER-HI DUPLICATS     
   EXEMPLE DE CRIDA:     PKG_ABM_PREUS.ALTA_PREUS (254,1,50,RSP);
***********************************************************************************/
 PROCEDURE ALTA_PREUS (
                          a_idproducte    IN PREUS.IDPRODUCTE%TYPE,
                          a_idpais        IN PREUS.IDPAIS%TYPE,
                          a_preuaplic     IN PREUS.PREUAPLIC%TYPE,
                          a_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         MODIF_PREUS
   DESCRIPCI:          MODIFICA EL PREU D'UN PRODUCTE EN UN PAIS
                        DETERMINAT A LA TAULA PREUS                      
   PARAMETRES ENTRADA:  m_idpreus, m_idproducte, m_idpais, m_preuaplic
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI PREUS NO POT SER NULL
                        ERROR: PREUS NO EXISTEIX
                        ERROR: CLAU FORANA INCORRECTA
                        ERROR: VALOR NO POT SER NULL
                        ERROR: CLAU UNICA: PRODUCTE + PAIS, NO POT HAVER-HI DUPLICATS 
   EXEMPLE DE CRIDA:    PKG_ABM_PREUS.MODIF_PREUS (254,1,85,RSP);
***********************************************************************************/  
 PROCEDURE MODIF_PREUS (
                          m_idpreus       IN PREUS.IDPREUS%TYPE,
                          m_idproducte    IN PREUS.IDPRODUCTE%TYPE,
                          m_idpais        IN PREUS.IDPAIS%TYPE,
                          m_preuaplic     IN PREUS.PREUAPLIC%TYPE,
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         BAIXA_PREUS
   DESCRIPCI:          ESBORRA EL PREU D'UN PRODUCTE EN UN PAIS
                        DETERMINAT A LA TAULA PREUS 
   PARAMETRES ENTRADA:  b_idpreus   
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI PREUS NO POT SER NULL
                        ERROR: PREU NO EXISTEIX
   EXEMPLE DE CRIDA:    PKG_ABM_PREUS.BAIXA_PREUS (254,1,RSP);
***********************************************************************************/
PROCEDURE BAIXA_PREUS (
                        b_idpreus       IN PREUS.IDPREUS%TYPE,
                        b_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/* VARIABLES GLOBALS */
          v_prodediment      LOGS.PROCEDIMENT%TYPE;
          v_param_entrada    LOGS.PARAMIN%TYPE;         
          v_id               PREUS.IDPREUS%TYPE;
          v_Total            number;
          v_error            number;
          v_desc             VARCHAR2(256);

          EX_INS_NUL         EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
          EX_FK_NUL          EXCEPTION;   -- no existeix la clau forana
          EX_IDPREUS_NUL     EXCEPTION;  -- el codi de PREUS no pot ser null

    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

END PKG_ABM_PREUS;

/
--------------------------------------------------------
--  DDL for Package PKG_ABM_PRODUCTE
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_PRODUCTE" AS 
 
 /************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: PKG_ABM_PRODUCTE.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_PRODUCTE
   DESCRIPCI:          AFEGEIX LES DADES D'UN PRODUCTE PER DESCARREGAR A LA TAULA PRODUCTE,
                        POSA EN ESTAT INACTIU (I) LA VERSIO ANTERIOR DEL PRODUCTE (SI EXISTEIX)
                        , I ACTUALITZA L'HISTORIC DE VERSIONS
   PARAMETRES ENTRADA:  a_idaplicacio, a_idssoo,a_versio,a_enllafitxer,a_midaaplic,
                        a_resolucio, a_datapujada
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA: APLICACIO + SSOO + VERSIO, NO POT HAVER-HI DUPLICATS
                         ERROR: CLAU UNICA: ENLLA APLICACIO, NO POT HAVER-HI DUPLICATS                         
   EXEMPLE DE CRIDA:     PKG_ABM_PRODUCTE.ALTA_PRODUCTE (25,1,'1.2.0','https://play.google.com/store/apps/details?id=com.levelup.beautifulwidgets#?
                          t=W251bGwsMSwxLDIxMiwiY29tLmxldmVsdXAuYmVhdXRpZnVsd2lkZ2V0cyJd','5,8M',
                          '320 x 480 pixels','5/12/12',RSP);
***********************************************************************************/
 PROCEDURE ALTA_PRODUCTE (
                          a_idaplicacio   IN PRODUCTE.IDAPLIC%TYPE,
                          a_idssoo        IN PRODUCTE.IDSSOO%TYPE,
                          a_versio        IN PRODUCTE.VERSIO%TYPE,
                          a_enllafitxer  IN PRODUCTE.ENLLAFITXER%TYPE,      
                          a_midaaplic     IN PRODUCTE.MIDAAPLIC%TYPE,
                          a_resolucio     IN PRODUCTE.RESOLUCIO%TYPE,  
                          a_datapujada    IN PRODUCTE.DATAPUJADA%TYPE,
                          a_RSP           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         MODIF_PRODUCTE
   DESCRIPCI:          MODIFICA LES DADES D'UN PRODUCTE EXISTENT A LA TAULA PRODUCTE
   PARAMETRES ENTRADA:  m_idaplicacio, m_idssoo, m_versio, m_enllafitxer, 
                        m_midaaplic, m_resolucio
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: ELS PARAMETRES ENTRADA CODI APLICACIO, SSOO i VERSIO NO PODEN SER NULL
                        ERROR: PRODUCTE NO EXISTEIX
                        ERROR: CLAU FORANA INCORRECTA
                        ERROR: VALOR NO POT SER NULL
                        ERROR: CLAU UNICA: APLICACIO + SSOO + VERSIO, NO POT HAVER-HI DUPLICATS
                        ERROR: CLAU UNICA: ENLLA APLICACIO, NO POT HAVER-HI DUPLICATS 
   EXEMPLE DE CRIDA:    PKG_ABM_PRODUCTE.MODIF_PRODUCTE (25,1,'1.2.0','https://play.google.com/store/apps/details',
                        '5,8M','320 x 480 pixels',RSP);
***********************************************************************************/  
 PROCEDURE MODIF_PRODUCTE (
                          m_idaplicacio   IN PRODUCTE.IDAPLIC%TYPE,
                          m_idssoo        IN PRODUCTE.IDSSOO%TYPE,
                          m_versio        IN PRODUCTE.VERSIO%TYPE,
                          m_enllafitxer  IN PRODUCTE.ENLLAFITXER%TYPE,      
                          m_midaaplic     IN PRODUCTE.MIDAAPLIC%TYPE,
                          m_resolucio     IN PRODUCTE.RESOLUCIO%TYPE,  
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         BAIXA_PRODUCTE
   DESCRIPCI:          NO S'EFECTUA LA BAIXA LGICA DEL PRODUCTE, 
                        CANVIA L'ESTAT D'UN PRODUCTE A ESTAT D'INACTIU,
                        POSA EN ESTAT ACTIU (A) LA VERSIO ANTERIOR DEL PRODUCTE 
                        (SI EXISTEIX), I ACTUALITZA L'HISTORIC DE VERSIONS
   PARAMETRES ENTRADA:  b_idaplicacio, b_idssoo, b_versio
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: ELS PARAMETRES ENTRADA CODI APLICACIO, SSOO i VERSIO NO PODEN SER NULL
                        ERROR: PRODUCTE NO EXISTEIX
                        ERROR: PRODUCTE EN ESTAT INACTIU
   EXEMPLE DE CRIDA:    PKG_ABM_PRODUCTE.BAIXA_PRODUCTE (25,1,'1.2.0',RSP);
***********************************************************************************/
PROCEDURE BAIXA_PRODUCTE (
                          b_idaplicacio   IN PRODUCTE.IDAPLIC%TYPE,
                          b_idssoo        IN PRODUCTE.IDSSOO%TYPE,
                          b_versio        IN PRODUCTE.VERSIO%TYPE,
                          b_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/* VARIABLES GLOBALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_id                PRODUCTE.IDAPLIC%TYPE;
    v_idssoo            SSOO.IDSSOO%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_Total             number;
    v_Taplic            number;
    v_estat             PRODUCTE.ESTATVERSIO%TYPE;
    v_oldversio         PRODUCTE.CODIPRODUCTE%TYPE;
    v_newversio         PRODUCTE.CODIPRODUCTE%TYPE;
    v_test              number;
  
    EX_INS_NUL EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL EXCEPTION;   -- no existeix la clau forana
    EX_DATA_NO_OK EXCEPTION;   -- data no correcta
    EX_PRODUCTE_ACTIU EXCEPTION;   -- control de producte actiu (amb descarregues associades)
    EX_PRODUCTE_INACTIU EXCEPTION;  -- si el producte ja esta en estat de baixa
    EX_IDPRODUCTE_NUL       EXCEPTION;  -- el codi de l'APLICACIO + SSOO + VERSIO no poden ser null
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

END PKG_ABM_PRODUCTE;

/
--------------------------------------------------------
--  DDL for Package PKG_ABM_VIDEO
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "APPS"."PKG_ABM_VIDEO" AS 

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: PKG_ABM_VIDEO.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_VIDEO
   DESCRIPCI:          AFEGEIX UNA NOU VIDEO D'UN PRODUCTE EN UN IDIOMA 
                        DETERMINAT A LA TAULA VIDEO               
   PARAMETRES ENTRADA:  a_idproducte, a_ididioma, a_enllavideo
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: VALOR NO POT SER NULL
                         ERROR: CLAU UNICA:PRODUCTE + IDIOMA, NO POT HAVER-HI DUPLICATS   
   EXEMPLE DE CRIDA:     PKG_ABM_VIDEO.ALTA_VIDEO (254,2,' XXXXXX',RSP);
***********************************************************************************/
 PROCEDURE ALTA_VIDEO (
                          a_idproducte    IN VIDEO.IDPRODUCTE%TYPE,
                          a_ididioma      IN VIDEO.IDIDIOMA%TYPE,
                          a_enllavideo   IN VIDEO.ENLLAVIDEO%TYPE,
                          a_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         MODIF_VIDEO
   DESCRIPCI:          MODIFICA L'ENLLA D'UN VIDEO EN UN IDIOMA D'UN PRODUCTE
                        EXISTENT A LA TAULA VIDEO                  
   PARAMETRES ENTRADA:  m_idvideo, m_idproducte, m_ididioma, m_enllavideo
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI VIDEO NO POT SER NULL
                        ERROR: VIDEO NO EXISTEIX
                        ERROR: CLAU FORANA INCORRECTA
                        ERROR: VALOR NO POT SER NULL
                        ERROR: CLAU UNICA:PRODUCTE + IDIOMA, NO POT HAVER-HI DUPLICATS   
   EXEMPLE DE CRIDA:    PKG_ABM_VIDEO.MODIF_VIDEO (21,254,2,'CCCCCCC',RSP);
***********************************************************************************/  
 PROCEDURE MODIF_VIDEO (
                          m_idvideo       IN VIDEO.IDVIDEO%TYPE,
                          m_idproducte    IN VIDEO.IDPRODUCTE%TYPE,
                          m_ididioma      IN VIDEO.IDIDIOMA%TYPE,
                          m_enllavideo   IN VIDEO.ENLLAVIDEO%TYPE,
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/************************************************************************************
   PROCEDIMENT:         BAIXA_VIDEO
   DESCRIPCI:          ESBORRA L'ENLLA D'UN VIDEO EN UN IDIOMA D'UN PRODUCTE
                        EXISTENT A LA TAULA VIDEO  
   PARAMETRES ENTRADA:  b_idvideo
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI VIDEO NO POT SER NULL
                        ERROR: VIDEO NO EXISTEIX
   EXEMPLE DE CRIDA:    PKG_ABM_VIDEO.BAIXA_VIDEO(254,2,RSP);
***********************************************************************************/
PROCEDURE BAIXA_VIDEO (
                        b_idvideo       IN VIDEO.IDVIDEO%TYPE,
                        b_rsp           OUT NOCOPY LOGS.paramOut%TYPE);
          
/* VARIABLES GLOBALS */
          v_prodediment      LOGS.PROCEDIMENT%TYPE;
          v_id               VIDEO.IDVIDEO%TYPE;
          v_Total            number;
          v_param_entrada    LOGS.PARAMIN%TYPE; 
          v_error            number;
          v_desc             VARCHAR2(256);

          EX_INS_NUL         EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
          EX_FK_NUL          EXCEPTION;  -- no existeix la clau forana
          EX_IDVIDEO_NUL     EXCEPTION;  -- el codi del VIDEO no pot ser null

    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
   
  
END PKG_ABM_VIDEO;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_APLICACIONS
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_APLICACIONS" AS
 
  PROCEDURE ALTA_APLICACIO (
                            a_nomaplicacio  IN APLICACIO.NOMAPLI%TYPE,
                            a_idcategoria   IN APLICACIO.IDCATEGORIA%TYPE,
                            a_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
     v_prodediment :=   'PKG_ABM_ALTA_APLICACIO ';	
     v_param_entrada := 'NOM APLICACIO: '||a_nomaplicacio||
                        ' / CATEGORIA: '||a_idcategoria;
       
        -- INSERTEM LA  NOVA APLICACIO
          	INSERT INTO APLICACIO 
        	  VALUES (NULL,UPPER(a_nomaplicacio), a_idcategoria);
         
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

COMMIT;
      
EXCEPTION 
    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

    WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

    WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA: NOM APLICACIO, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
  END ALTA_APLICACIO;

  PROCEDURE MODIF_APLICACIO (
                              m_idaplicacio   IN APLICACIO.IDAPLICACIO%TYPE,
                              m_nomaplicacio  IN APLICACIO.NOMAPLI%TYPE,
                              m_idcategoria   IN APLICACIO.IDCATEGORIA%TYPE,
                              m_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  
  BEGIN
     v_prodediment :=   'PKG_ABM_MODIF_APLICACIO ';	
     v_param_entrada := 'IDENTIFICADOR APLICACIO: ' ||m_idaplicacio||
                        ' / NOM APLICACIO: '        ||m_nomaplicacio||
                        ' / CATEGORIA: '            ||m_idcategoria;
                        
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_idaplicacio NO SIGUI NULL
            IF (m_idaplicacio IS NULL ) THEN
                  RAISE EX_IDAPLIC_NUL;
            END IF;
            
     -- VERIFIQUEM QUE L'APLICACIO EXISTEIXI A LA TAULA APLICACIO
            SELECT IDAPLICACIO  INTO  v_id
                 FROM   APLICACIO
                        WHERE  IDAPLICACIO = m_idaplicacio;
                        
    -- VERIFIQUEM SI L'APLICACIO HA CANVIAT DE NOM (per actualitzar les estadistiques)
           SELECT NOMAPLI INTO v_NOM_old
                 FROM    APLICACIO
                       WHERE  IDAPLICACIO = m_idaplicacio;                
      
     -- MODIFIQUEM LES DADES DE L'APLICACIO
            UPDATE APLICACIO
               SET NOMAPLI =         UPPER(m_nomaplicacio),
                  IDCATEGORIA =     m_idcategoria
                         WHERE IDAPLICACIO = m_idaplicacio;
          
      -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
COMMIT;
 
      -- SI L'APLICACIO HA CANVIAT DE NOM
      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
      
            IF (UPPER(v_NOM_old) <> UPPER(m_nomaplicacio)) THEN
                MODUL_EST_345(extract(year from SYSDATE),m_rsp);   
            END IF;
COMMIT;

EXCEPTION 
          
    WHEN EX_IDAPLIC_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI APLICACIO NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
          
    WHEN NO_DATA_FOUND THEN  -- CONTROL QUE L'APLICACIO EXISTEIXI A LA TAULA APLICACIO
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: APLICACIO NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
    WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

    WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
  END MODIF_APLICACIO;
  
  PROCEDURE BAIXA_APLICACIO (
                             b_idaplicacio   IN APLICACIO.IDAPLICACIO%TYPE,
                             b_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
    v_prodediment :=    'PKG_ABM_BAIXA_APLICACIO ';	
    v_param_entrada :=  'IDENTIFICADOR APLICACIO: '||b_idaplicacio;
     

     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA b_idaplicacio NO SIGUI NULL
            IF (b_idaplicacio IS NULL ) THEN
                  RAISE EX_IDAPLIC_NUL;
            END IF;

     -- VERIFIQUEM QUE L'APLICACIO EXISTEIXI A LA TAULA APLICACIO
            SELECT IDAPLICACIO  INTO   v_id
                 FROM   APLICACIO
                        WHERE  IDAPLICACIO = b_idaplicacio;
  
      -- VERIFIQUEM QUE L'APLICACIO NO ESTIGUI ASSOCIADA A CAP PRODUCTE
            SELECT COUNT(IDAPLIC) INTO v_Total
                 FROM PRODUCTE
                       WHERE IDAPLIC = b_idaplicacio;
        
            IF (v_Total > 0) THEN
                 RAISE EX_APLICACIO_ACTIU;
            END IF;
   
      -- ESBORREM LES DADES DE L'APLICACIO DE LA TAULA APLICACIO
           DELETE FROM APLICACIO WHERE IDAPLICACIO = b_idaplicacio;    
      
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
COMMIT;

EXCEPTION 
   
    WHEN EX_IDAPLIC_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI APLICACIO NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

    WHEN NO_DATA_FOUND THEN     -- CONTROL QUE L'APLICACIO EXISTEIXI A LA TAULA APLICACIO
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: APLICACIO NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
    WHEN EX_APLICACIO_ACTIU THEN   -- CONTROL L'APLICACIO NO ESTIGUI ASSOCIADA A CAP PRODUCTE
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: APLICACIO ASSOCIADA A PRODUCTE '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
              
  END BAIXA_APLICACIO;
  
END PKG_ABM_APLICACIONS;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_AUTOR
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_AUTOR" AS

  PROCEDURE ALTA_AUTOR (
                          a_idprod       IN AUTOR.IDPROD%TYPE,
                          a_iddesev      IN AUTOR.IDDESEV%TYPE,
                          a_rsp          OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
 v_prodediment :=     'PKG_ABM_ALTA_AUTOR ';	
 v_param_entrada :=   'PRODUCTE NUM: '          ||a_idprod||
                      ' / CODI DESENVOLUPADOR: '||a_iddesev;
       
        -- INSERTEM EL NOU AUTOR D'UNA APLICACIO EN UN SSOO DETERMINAT 
        -- I UNA VERSIO DETERMINADA A LA TAULA AUTOR  
           	INSERT INTO AUTOR 
                VALUES (NULL, a_idprod,a_iddesev);
         
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

 COMMIT;

       -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
             MODUL_EST_345(extract(year from SYSDATE),a_rsp);   
 COMMIT;  
 
 EXCEPTION 
      WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

       WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

       WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA: PRODUCTE + DESENVOLUPADOR, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

        WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
  END ALTA_AUTOR;

  PROCEDURE MODIF_AUTOR (
                          m_idautor       IN AUTOR.IDAUTOR%TYPE,
                          m_idprod        IN AUTOR.IDPROD%TYPE,
                          m_iddesev       IN AUTOR.IDDESEV%TYPE,
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
     v_prodediment :=   'PKG_ABM_MODIF_AUTOR ';	
     v_param_entrada := 'IDENTIF. AUTOR: '        ||m_idautor||
                        ' / PRODUCTE NUM.: '      ||m_idprod||
                        ' / CODI DESENVOLUPADOR: '||m_iddesev;
                        
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_idautor NO SIGUI NULL
            IF (m_idautor IS NULL ) THEN
                  RAISE EX_IDAUTOR_NUL;
            END IF;
                                    
     -- VERIFIQUEM QUE L'AUTOR EXISTEIXI A LA TAULA AUTOR
          SELECT IDAUTOR  INTO   v_id
                 FROM   AUTOR
                        WHERE  IDAUTOR =  m_idautor;
      
     -- MODIFIQUEM L'AUTOR
          UPDATE  AUTOR
              SET 
                  IDPROD = m_idprod,
                  IDDESEV = m_iddesev
                        WHERE  IDAUTOR =  m_idautor;
          
      -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
 COMMIT;

      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
             MODUL_EST_345(extract(year from SYSDATE),m_rsp);      
 COMMIT;
 
 EXCEPTION 
          
     WHEN EX_IDAUTOR_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI AUTOR NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
     WHEN NO_DATA_FOUND THEN  -- CONTROL QUE EL AUTOR EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL AUTOR NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
     WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

     WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA: PRODUCTE + DESENVOLUPADOR, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

     WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
  END MODIF_AUTOR;

  PROCEDURE BAIXA_AUTOR (
                          b_idautor       IN AUTOR.IDAUTOR%TYPE,
                          b_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
    v_prodediment :=    'PKG_ABM_BAIXA_AUTOR ';	
    v_param_entrada :=  'CODI AUTOR: '         ||b_idautor;
                        
                        
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA b_idautor NO SIGUI NULL
            IF (b_idautor IS NULL ) THEN
                  RAISE EX_IDAUTOR_NUL;
            END IF;
            
      -- VERIFIQUEM QUE L'AUTOR EXISTEIXI A LA TAULA AUTOR
          SELECT IDAUTOR  INTO   v_id
                 FROM   AUTOR
                        WHERE  IDAUTOR = b_idautor ;
     
     
      -- ESBORREM L'AUTOR
          DELETE 
                FROM AUTOR 
                       WHERE  IDAUTOR = b_idautor ;
      
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
COMMIT;

      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
             MODUL_EST_345(extract(year from SYSDATE),b_rsp);  
COMMIT;
 
EXCEPTION 

     WHEN EX_IDAUTOR_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI AUTOR NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

    WHEN NO_DATA_FOUND THEN     -- CONTROL QUE L'AUTOR EXISTEIXI A LA TAULA AUTOR
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: AUTOR NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
                 
     WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
  END BAIXA_AUTOR;

END PKG_ABM_AUTOR;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_DESCRIP
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_DESCRIP" AS

  PROCEDURE ALTA_DESCRIP (
                          a_idaplicacio   IN DESCRIPCIO.IDAPLICACIO%TYPE,
                          a_ididioma      IN DESCRIPCIO.IDIDIOMA%TYPE,
                          a_descrip       IN DESCRIPCIO.DESCRIPCIO%TYPE,
                          a_rsp           OUT NOCOPY LOGS.paramOut%TYPE) 
  AS                
  BEGIN
  
   v_prodediment :=     'PKG_ABM_ALTA_DESCRIP ';	
   v_param_entrada :=   'APLICACIO NUM: ' ||a_idaplicacio||
                        ' / IDIOMA: '     ||a_ididioma||
                        ' / DESCRIPCIO: ' ||SUBSTR(a_descrip, 1, 100);
       
        -- INSERTEM LA  NOVA APLICACIO
          	INSERT INTO DESCRIPCIO 
        	  VALUES (NULL, a_idaplicacio,a_ididioma, UPPER(a_descrip));
       
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

  COMMIT;
      
  EXCEPTION 
  
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA: APLICACIO + IDIOMA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
   
   
  END ALTA_DESCRIP;

  PROCEDURE MODIF_DESCRIP (
                            m_iddescrip     IN DESCRIPCIO.IDDESCRIPCIO%TYPE,
                            m_idaplicacio   IN DESCRIPCIO.IDAPLICACIO%TYPE,
                            m_ididioma      IN DESCRIPCIO.IDIDIOMA%TYPE,
                            m_descrip       IN DESCRIPCIO.DESCRIPCIO%TYPE,
                            m_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
     v_prodediment :=   'PKG_ABM_MODIF_DESCRIP ';	
     v_param_entrada := 'IDENTIF. DESCRIPCIO: '||m_iddescrip||
                        ' / APLICACIO NUM.: '  ||m_idaplicacio||
                        ' / IDIOMA: '          ||m_ididioma||
                        ' / DESCRIPCIO: '      ||SUBSTR(m_descrip, 1, 100);

     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_iddescrip NO SIGUI NULL
            IF (m_iddescrip IS NULL ) THEN
                  RAISE EX_IDDESCR_NUL;
            END IF;
                        
     -- VERIFIQUEM QUE LA DESCRIPCIO EXISTEIXI A LA TAULA DESCRIPCIO
          SELECT IDDESCRIPCIO  INTO   v_id
                 FROM   DESCRIPCIO
                        WHERE  IDDESCRIPCIO = m_iddescrip;
      
     -- MODIFIQUEM LA DESCRIPCIO DE L'APLICACI EN UN IDIOMA DETERMINAT
          UPDATE  DESCRIPCIO
              SET IDAPLICACIO =   m_idaplicacio,
                  IDIDIOMA =      m_ididioma,
                  DESCRIPCIO =    UPPER(m_descrip) 
                        WHERE IDDESCRIPCIO = m_iddescrip;
          
      -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
  COMMIT;
   
  EXCEPTION 

       WHEN EX_IDDESCR_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI DESCRIPCIO NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
          
           WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
          WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA: APLICACIO + IDIOMA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
           
           WHEN NO_DATA_FOUND THEN  -- CONTROL QUE LA DESCRIPCIO EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: LA DESCRIPCIO NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
  END MODIF_DESCRIP;

  PROCEDURE BAIXA_DESCRIP (
                            b_iddescrip     IN DESCRIPCIO.IDDESCRIPCIO%TYPE,
                            b_rsp           OUT NOCOPY LOGS.paramOut%TYPE) 
  AS
  BEGIN
    v_prodediment :=    'PKG_ABM_BAIXA_DESCRIP ';	
    v_param_entrada :=  'DESCRIPCIO NUM.: '||b_iddescrip;

     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA b_iddescrip NO SIGUI NULL
            IF (b_iddescrip IS NULL ) THEN
                  RAISE EX_IDDESCR_NUL;
            END IF;
                        
    -- VERIFIQUEM QUE LA DESCRIPCIO EXISTEIXI A LA TAULA DESCRIPCIO
          SELECT IDDESCRIPCIO  INTO   v_id
                 FROM   DESCRIPCIO
                       WHERE  IDDESCRIPCIO = b_iddescrip;
     
    -- ESBORREM LA DESCRIPCIO DE L'APLICACIO EN UN IDIOMA DETERMINAT DE LA TAULA DESCRIPCIO
          DELETE 
                FROM DESCRIPCIO 
                        WHERE  IDDESCRIPCIO = b_iddescrip;
      
    -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
    -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
 COMMIT;

 EXCEPTION 
   
        WHEN EX_IDDESCR_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI DESCRIPCIO NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
          
         WHEN NO_DATA_FOUND THEN     -- CONTROL QUE LA DESCRIPCIO EXISTEIXI A LA TAULA DESCRIPCIO
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: LA DESCRIPCIO NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
                 
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
  END BAIXA_DESCRIP;

END PKG_ABM_DESCRIP;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_DISPOSITIU
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_DISPOSITIU" AS
 
  PROCEDURE ALTA_DISPOSITIU (
                          a_idusuari      IN DISPOSITIU.IDUSUARI%TYPE,
                          a_codiimei      IN DISPOSITIU.CODIIMEI%TYPE,
                          a_idmodel       IN DISPOSITIU.IDMODEL%TYPE,
                          a_idSSOO        IN DISPOSITIU.IDSSOO%TYPE,
                          a_versiossoo    IN DISPOSITIU.VERSIOSSOO%TYPE,      
                          a_resolucio     IN DISPOSITIU.RESOLUCIO%TYPE,
                          a_ididiomadisp  IN DISPOSITIU.IDIDIOMADISP%TYPE,          
                          a_RSP           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
   v_prodediment := 'PKG_ABM_ALTA_DISPOSITIU ';	
   v_param_entrada :=   'USUARI NUM: '    ||a_idusuari||
                        ' / CODI IMEI: '  ||a_codiimei||
                        ' / CODI MODEL: ' ||a_idmodel||
                        ' / CODI SSOO: '  ||a_idSSOO||
                        ' / VERSIO SSOO: '||TRIM(a_versiossoo)||
                        ' / RESOLUCIO: '  ||TRIM(a_resolucio)||
                        ' / CODI IDIOMA: '||a_ididiomadisp;

       -- VERIFIQUEM QUE EL CODI IMEI TINGUI 15 DIGITS
            IF (LENGTH(a_codiimei) <> 15) THEN
                RAISE EX_IMEI_NOK;
            END IF;
         
        -- INSERTEM EL NOU PRODUCTE
          	INSERT INTO DISPOSITIU 
        	  VALUES (NULL, a_idusuari, UPPER(a_codiimei), a_idmodel, a_idSSOO, UPPER(a_versiossoo),
                          UPPER(a_resolucio), a_ididiomadisp, null, Null );
         
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

       COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA: CODI IMEI, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
          WHEN EX_IMEI_NOK THEN   -- CONTROL QUE EL CODI IMEI TINGUI 15 DIGITS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: EL EL CODI IMEI TINGUI 15 DIGITS '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);     
               
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
  END ALTA_DISPOSITIU;

  PROCEDURE MODIF_DISPOSITIU (
                          m_iddispositiu    IN DISPOSITIU.IDDISPOSITIU%TYPE,
                          m_idusuari        IN DISPOSITIU.IDUSUARI%TYPE,
                          m_codiimei        IN DISPOSITIU.CODIIMEI%TYPE,
                          m_idmodel         IN DISPOSITIU.IDMODEL%TYPE,
                          m_idSSOO          IN DISPOSITIU.IDSSOO%TYPE,
                          m_versiossoo      IN DISPOSITIU.VERSIOSSOO%TYPE,      
                          m_resolucio       IN DISPOSITIU.RESOLUCIO%TYPE,
                          m_ididiomadisp    IN DISPOSITIU.IDIDIOMADISP%TYPE,  
                          m_estat           IN DISPOSITIU.ESTAT%TYPE, 
                          m_rsp             OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
     v_prodediment := 'PKG_ABM_MODIF_DISPOSITIU ';	
     v_param_entrada := 'DISPOSITIU NUM: '||m_iddispositiu||
                        ' / USUARI NUM: ' ||m_idusuari||
                        ' / CODI IMEI: '  ||m_codiimei||
                        ' / CODI MODEL: ' ||m_idmodel||
                        ' / CODI SSOO: '  ||m_idSSOO||
                        ' / VERSIO SSOO: '||TRIM(m_versiossoo)||
                        ' / RESOLUCIO: '  ||TRIM(m_resolucio)||
                        ' / CODI IDIOMA: '||m_ididiomadisp||                        
                        ' / ESTAT: '      ||m_estat;
     
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_iddispositiu NO SIGUI NULL
            IF (m_iddispositiu IS NULL ) THEN
                  RAISE EX_IDDISP_NUL;
            END IF;
               
    -- VERIFIQUEM QUE EL DISPOSITIU EXISTEIXI A LA TAULA DISPOSITIU
            SELECT IDDISPOSITIU  INTO   v_id
                    FROM   DISPOSITIU
                        WHERE  IDDISPOSITIU = m_iddispositiu;
 
    -- VERIFIQUEM QUE m_estat = 'A', en cas que sigui 'B' DONAR ERROR, PER DONAR DE BAIXA
    -- UN DISPOSITIU S'HA D'ANAR PER BAIXA_DISPOSITIU
            IF (UPPER(m_estat) <> 'A') THEN
                RAISE EX_EST_NOK;
            END IF;
        
    -- VERIFIQUEM QUE EL CODI IMEI TINGUI 15 DIGITS
            IF (LENGTH(m_codiimei) <> 15) THEN
                RAISE EX_IMEI_NOK;
            END IF;   
            
     -- MODIFIQUEM EL DISPOSITIU 
            UPDATE  DISPOSITIU
               SET 
                  IDUSUARI =        m_idusuari,
                  CODIIMEI =        m_codiimei,
                  IDMODEL =         m_idmodel,
                  IDSSOO =          m_idSSOO,
                  VERSIOSSOO =      UPPER(m_versiossoo),
                  RESOLUCIO =       UPPER(m_resolucio),
                  IDIDIOMADISP =    m_ididiomadisp,
                  ESTAT =           UPPER(m_estat),
                  DATAESTAT =       SYSDATE
               WHERE IDDISPOSITIU = m_iddispositiu;
 
    -- FINALITZA AMB EXIT
            m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
    -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
   COMMIT;
   
 EXCEPTION 
          
          WHEN EX_IDDISP_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI DISPOSITIU NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
          
          WHEN NO_DATA_FOUND THEN  -- CONTROL QUE EL DISPOSITIU EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL DISPOSITIU NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
          WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA: CODI IMEI, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

          WHEN EX_EST_NOK THEN   -- CONTROL L'ESTAT NOMS POT SER 'A'
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: ESTAT NOMS POT SER A (ALTA) '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);  

          WHEN EX_IMEI_NOK THEN   -- CONTROL QUE EL CODI IMEI TINGUI 15 DIGITS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL EL CODI IMEI TINGUI 15 DIGITS '||v_error|| ': '||v_desc;
             
           -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);  

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
  END MODIF_DISPOSITIU;

  PROCEDURE BAIXA_DISPOSITIU (
                                b_iddispositiu   IN DISPOSITIU.IDDISPOSITIU%TYPE,
                                b_rsp            OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
   v_prodediment :=    'PKG_ABM_BAIXA_DISPOSITIU ';	
   v_param_entrada :=  'DISPOSITIU NUM: '||b_iddispositiu;
    
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_iddispositiu NO SIGUI NULL
            IF (b_iddispositiu IS NULL ) THEN
                  RAISE EX_IDDISP_NUL;
            END IF;
            
    -- VERIFIQUEM QUE EL DISPOSITIU EXISTEIXI A LA TAULA DISPOSITIU
          SELECT IDDISPOSITIU  INTO   v_id
                 FROM   DISPOSITIU
                        WHERE  IDDISPOSITIU = b_iddispositiu;

    -- VERIFIQUEM QUE EL DISPOSITIU NO ESTIGUI JA EN ESTAT DE BAIXA
          SELECT ESTAT  INTO   v_estat
                 FROM   DISPOSITIU
                        WHERE  IDDISPOSITIU = b_iddispositiu;
          IF (UPPER(v_estat) ='B') THEN
                 RAISE EX_DISPOSITIU_INACTIU;
          END IF;

     -- VERIFIQUEM QUE EL DISPOSITIU NO TINGUI ASSOCIADA CAP DESCARREGA
          SELECT COUNT(IDDISPOSITIU) INTO v_Total
                 FROM DESCARREGA
                       WHERE IDDISPOSITIU = b_iddispositiu;
          IF (v_Total > 0)  THEN
              -- CANVIEM L'ESTAT A BAIXA
                    UPDATE DISPOSITIU
                        SET ESTAT     = 'B',
                            DATAESTAT =  SYSDATE
                          WHERE IDDISPOSITIU = b_iddispositiu;       
           ELSE
              -- ESBORREM LES DADES DEL DISPOSITIU DE LA TAULA DISPOSITIU
                    DELETE 
                        FROM DISPOSITIU 
                            WHERE IDDISPOSITIU = b_iddispositiu;                   
          END IF;

      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
 COMMIT;

  EXCEPTION 

          WHEN EX_IDDISP_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI DISPOSITIU NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

         WHEN NO_DATA_FOUND THEN     -- CONTROL QUE EL DISPOSITIU EXISTEIXI A LA TAULA DISPOSITIU
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL DISPOSITIU NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
   
          WHEN EX_DISPOSITIU_INACTIU THEN   -- CONTROL QUE EL DISPOSITIU NO TINGUI JA L'ESTAT INACTIU
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: DISPOSITIU INACTIU '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);  
               
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
  END BAIXA_DISPOSITIU;

END PKG_ABM_DISPOSITIU;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_PREUS
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_PREUS" AS

  PROCEDURE ALTA_PREUS (
                          a_idproducte    IN PREUS.IDPRODUCTE%TYPE,
                          a_idpais        IN PREUS.IDPAIS%TYPE,
                          a_preuaplic     IN PREUS.PREUAPLIC%TYPE,
                          a_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
  v_prodediment :=      'PKG_ABM_ALTA_PREUS ';	
  v_param_entrada :=    'PRODUCTE NUM: '||a_idproducte||
                        ' / PAIS: '     ||a_idpais||
                        ' / PREU: '     ||a_preuaplic ;
       
        -- INSERTEM EL NOU PREU D'UN PRODUCTE EN UN PAIS DETERMINAT
          	INSERT INTO PREUS 
                VALUES (NULL, a_idproducte, a_idpais, a_preuaplic, null);
         
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

COMMIT;
        
EXCEPTION 
       WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

       WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

       WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA: PRODUCTE + PAIS, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

  END ALTA_PREUS;

  PROCEDURE MODIF_PREUS (
                          m_idpreus       IN PREUS.IDPREUS%TYPE,
                          m_idproducte    IN PREUS.IDPRODUCTE%TYPE,
                          m_idpais        IN PREUS.IDPAIS%TYPE,
                          m_preuaplic     IN PREUS.PREUAPLIC%TYPE,
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
       v_prodediment :=   'PKG_ABM_MODIF_PREUS ';	
       v_param_entrada := 'IDENTIF. PREUS : '||m_idpreus||
                        ' / PRODUCTE NUM.: ' ||m_idproducte||
                        ' / PAIS: '          ||m_idpais||
                        ' / PREU: '          ||m_preuaplic;
 
      -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_idpreus NO SIGUI NULL
            IF (m_idpreus IS NULL ) THEN
                  RAISE EX_IDPREUS_NUL;
            END IF;
                                   
     -- VERIFIQUEM QUE EL PREU EXISTEIXI A LA TAULA PREUS
            SELECT IDPREUS  INTO   v_id
                 FROM   PREUS
                        WHERE  IDPREUS = m_idpreus;
      
     -- MODIFIQUEM EL PREU D'UN PRODUCTE EN UN PAIS DETERMINAT
            UPDATE  PREUS
               SET 
                  IDPRODUCTE = m_idproducte,
                  IDPAIS = m_idpais,
                  PREUAPLIC = m_preuaplic
                        WHERE  IDPREUS = m_idpreus;
          
      -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
COMMIT;
   
EXCEPTION 

    WHEN EX_IDPREUS_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI PREUS NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
                    
    WHEN NO_DATA_FOUND THEN  -- CONTROL QUE EL PREU EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PREU NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
     WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

     WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA: PRODUCTE + PAIS, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

     WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

  
  END MODIF_PREUS;

  PROCEDURE BAIXA_PREUS (
                        b_idpreus       IN PREUS.IDPREUS%TYPE,
                        b_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
    v_prodediment :=    'PKG_ABM_BAIXA_PREUS ';	
    v_param_entrada :=  'CODI PREUS: ' ||b_idpreus;
                        
      -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA b_idpreus NO SIGUI NULL
            IF (b_idpreus IS NULL ) THEN
                  RAISE EX_IDPREUS_NUL;
            END IF;
            
     -- VERIFIQUEM QUE EL PREU EXISTEIXI A LA TAULA PREUS
          SELECT IDPREUS  INTO   v_id
                 FROM   PREUS
                        WHERE  IDPREUS = b_idpreus;
     
      -- ESBORREM EL PREU D'UN PRODUCTE EN UN PAIS DETERMINAT
          DELETE 
                FROM PREUS 
                        WHERE  IDPREUS = b_idpreus;
      
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
COMMIT;

EXCEPTION 

     WHEN EX_IDPREUS_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI PREUS NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

     WHEN NO_DATA_FOUND THEN     -- CONTROL QUE EL PREU EXISTEIXI A LA TAULA PREUS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PREU NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);          
                
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
  END BAIXA_PREUS;

END PKG_ABM_PREUS;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_PRODUCTE
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_PRODUCTE" AS

  PROCEDURE ALTA_PRODUCTE (
                            a_idaplicacio   IN PRODUCTE.IDAPLIC%TYPE,
                            a_idssoo        IN PRODUCTE.IDSSOO%TYPE,
                            a_versio        IN PRODUCTE.VERSIO%TYPE,
                            a_enllafitxer  IN PRODUCTE.ENLLAFITXER%TYPE,      
                            a_midaaplic     IN PRODUCTE.MIDAAPLIC%TYPE,
                            a_resolucio     IN PRODUCTE.RESOLUCIO%TYPE,  
                            a_datapujada    IN PRODUCTE.DATAPUJADA%TYPE,
                            a_RSP           OUT NOCOPY LOGS.paramOut%TYPE) 
  AS

  BEGIN
   v_prodediment := 'PKG_ABM_ALTA_PRODUCTE ';	
   v_param_entrada :=  TRIM(SUBSTR('APLICACIO NUM: '||a_idaplicacio||
                        ' / SSOO: '                 ||a_idssoo||
                        ' / VERSIO: '               ||TRIM(a_versio)||
                        ' / ENLLA: '               ||SUBSTR(a_enllafitxer, 1, 100)||
                        ' / MIDA: '                 ||TRIM(a_midaaplic)||
                        ' / RESOLUCIO: '            ||TRIM(a_resolucio),1,1000)||
                        ' / DATA PUJADA: '          ||a_datapujada);
        
        -- MIREM SI EXISTEIX UNA VERSIO ANTERIOR 
            SELECT COUNT(*) INTO v_Taplic
                  FROM PRODUCTE
                    WHERE IDAPLIC= a_idaplicacio AND IDSSOO = a_idssoo;
                    
       -- BUSQUEM LA VERSIO ANTERIOR
            IF (v_Taplic > 0) THEN
              SELECT CODIPRODUCTE INTO v_oldversio
                  FROM PRODUCTE
                      WHERE IDAPLIC= a_idaplicacio AND IDSSOO = a_idssoo AND ESTATVERSIO = 'A';
            END IF;
           
       -- INSERTEM EL NOU PRODUCTE
          	 INSERT INTO PRODUCTE 
        	   VALUES (NULL, a_idaplicacio,a_idssoo, UPPER(a_versio), a_enllafitxer, 
                    a_midaaplic, a_datapujada, a_resolucio, NULL);
            COMMIT;
                    
       -- OBTENIM EL CODI DE PRODUCTE CORRESPONENT A LA NOVA VERSIO QUE HEM INSERTAT
            SELECT CODIPRODUCTE INTO v_newversio
                  FROM PRODUCTE
                    WHERE IDAPLIC= a_idaplicacio AND IDSSOO = a_idssoo AND VERSIO = a_versio;
                    
      -- REGISTREM LA NOVA VERSIO A HISTORIC DE VERSIONS              
            INSERT INTO VERSIONS_HIST
              VALUES (null, v_newversio, a_datapujada, null);  
                   
       -- SI EXISTIA UNA VERSIO ANTERIOR:
            IF (v_Taplic > 0) THEN
            -- A LA TAULA PRODUCTE LI CANVIEM L'ESTAT A INACTIVA   
                  UPDATE PRODUCTE 
                    SET ESTATVERSIO = 'I'
                        WHERE CODIPRODUCTE = v_oldversio;
            
            -- HO REGISTREM A HISTORIC DE VERSIONS
                  UPDATE VERSIONS_HIST
                    SET DATAFI = a_datapujada
                        WHERE IDPRODUCTE = v_oldversio;
            END IF;
          
       -- FINALITZA AMB EXIT
            a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

       COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAUS UNIQUES: APLICACIO + SSOO + VERSIO / ENLLA APLICACIO, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
          WHEN NO_DATA_FOUND THEN  -- CONTROL VALORS INCORRECTES 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: VALOR INCORRECTE '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
    
  END ALTA_PRODUCTE;

  PROCEDURE MODIF_PRODUCTE (
                          m_idaplicacio   IN PRODUCTE.IDAPLIC%TYPE,
                          m_idssoo        IN PRODUCTE.IDSSOO%TYPE,
                          m_versio        IN PRODUCTE.VERSIO%TYPE,
                          m_enllafitxer  IN PRODUCTE.ENLLAFITXER%TYPE,      
                          m_midaaplic     IN PRODUCTE.MIDAAPLIC%TYPE,
                          m_resolucio     IN PRODUCTE.RESOLUCIO%TYPE,  
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE) 
  AS

  BEGIN
  
     v_prodediment :=   'PKG_ABM_MODIF_PRODUCTE ';	
     v_param_entrada :=  TRIM(SUBSTR('APLICACIO NUM: '||m_idaplicacio||
                        ' / SSOO: '                   ||m_idssoo||
                        ' / VERSIO: '                 ||TRIM(m_versio)||
                        ' / ENLLA: '                 ||SUBSTR(m_enllafitxer, 1, 200)||
                        ' / MIDA: '                   ||TRIM(m_midaaplic)||
                        ' / RESOLUCIO: '              ||TRIM(m_resolucio),1,1000));
                        
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_idaplicacio, m_idssoo, m_versio NO SIGUIN NULL
            IF (m_idaplicacio IS NULL OR m_idssoo IS NULL OR m_versio IS NULL ) THEN
                  RAISE EX_IDPRODUCTE_NUL;
            END IF;
                                 
     -- VERIFIQUEM QUE EL PRODUCTE  EXISTEIXI A LA TAULA PRODUCTE
          SELECT CODIPRODUCTE  INTO   v_id
                 FROM   PRODUCTE
                        WHERE  IDAPLIC = m_idaplicacio AND IDSSOO = m_idssoo AND VERSIO = m_versio;
        
     -- MODIFIQUEM EL PRODUCTE CORRESPONENT A UNA APLICACI, SSOO I VERSIO DETERMINADA
          UPDATE  PRODUCTE
              SET 
                  ENLLAFITXER = m_enllafitxer,
                  MIDAAPLIC =    m_midaaplic,
                  RESOLUCIO =    m_resolucio
              WHERE IDAPLIC = m_idaplicacio AND IDSSOO = m_idssoo AND VERSIO = m_versio;
         
      -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
 COMMIT;
  
 EXCEPTION 

       WHEN EX_IDPRODUCTE_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: ELS PARAMETRES ENTRADA CODI APLICACIO, SSOO i VERSIO NO PODEN SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);        
               
       WHEN NO_DATA_FOUND THEN  -- CONTROL QUE EL PRODUCTE EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
       WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

       WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAUS UNIQUES: APLICACIO + SSOO + VERSIO / ENLLA APLICACIO, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

       WHEN  EX_DATA_NO_OK THEN  -- CONTROL DADES INCORRECTES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: DADES INCORRECTES '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

       WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
 
  END MODIF_PRODUCTE;

  PROCEDURE BAIXA_PRODUCTE (
                          b_idaplicacio   IN PRODUCTE.IDAPLIC%TYPE,
                          b_idssoo        IN PRODUCTE.IDSSOO%TYPE,
                          b_versio        IN PRODUCTE.VERSIO%TYPE,
                          b_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
     v_prodediment :=    'PKG_ABM_BAIXA_PRODUCTE ';	
     v_param_entrada :=  ' / APLICACIO NUM: '||b_idaplicacio||
                         ' / SSOO: '         ||b_idssoo||
                         ' / VERSIO: '       ||TRIM(b_versio);

     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA b_idaplicacio, b_idssoo, b_versio NO SIGUIN NULL
            IF (b_idaplicacio IS NULL OR b_idssoo IS NULL OR b_versio IS NULL ) THEN
                  RAISE EX_IDPRODUCTE_NUL;
            END IF;
    
      -- VERIFIQUEM QUE EL PRODUCTE EXISTEIXI A LA TAULA PRODUCTE 
          SELECT CODIPRODUCTE  INTO v_id
                 FROM   PRODUCTE
                      WHERE  IDAPLIC = b_idaplicacio AND IDSSOO = b_idssoo 
                              AND VERSIO = b_versio;

    -- VERIFIQUEM QUE EL PRODUCTE NO ESTIGUI JA EN ESTAT DE BAIXA (INACTIU)
          SELECT ESTATVERSIO  INTO   v_estat
                 FROM   PRODUCTE
                      WHERE  IDAPLIC = b_idaplicacio AND IDSSOO = b_idssoo 
                              AND VERSIO = b_versio;
           IF (UPPER(v_estat) ='I') THEN
                 RAISE EX_PRODUCTE_INACTIU;
          END IF;
    
    -- VERIFIQUEM QUE EL PRODUCTE TINGUI VERSIONS ANTERIORS
    
        SELECT COUNT( IDPRODUCTE) INTO v_test
                FROM VERSIONS_HIST 
                WHERE (IDPRODUCTE IN (SELECT CODIPRODUCTE 
                                        FROM PRODUCTE
                                           WHERE IDAPLIC = b_idaplicacio AND IDSSOO = b_idssoo));
    
      -- NO HI HA VERSIONS ANTERIORS D'AQUEST PRODUCTE:
          IF (v_test = 1 ) THEN
                -- HO REGISTREM A HISTORIC DE VERSIONS
                    UPDATE VERSIONS_HIST
                          SET DATAFI = sysdate
                              WHERE IDPRODUCTE = v_id AND DATAFI IS NULL;
              
                -- CANVIEM L'ESTAT A INACTIU AL PRODUCTE DONAT DE BAIXA
                    UPDATE PRODUCTE
                          SET ESTATVERSIO='I'
                              WHERE CODIPRODUCTE = v_id;
          ELSE
    --  HI HA MS D'UNA VERSIO ANTERIOR D'AQUEST PRODUCTE
        -- CERQUEM QUINA VA SER LA ANTERIOR VERSIO QUE VA ESTAR ACTIVA
                SELECT  IDPRODUCTE INTO v_oldversio
                  FROM VERSIONS_HIST 
                      WHERE DATAFI = (SELECT MAX(DATAFI) FROM versions_hist)
                            AND (IDPRODUCTE IN (SELECT CODIPRODUCTE 
                                                   FROM PRODUCTE
                                                      WHERE IDAPLIC = b_idaplicacio AND IDSSOO = b_idssoo));

         -- CANVIEM L'ESTAT DE LA VERSIO ANTERIOR A ACTIU      
              UPDATE PRODUCTE 
                 SET ESTATVERSIO = 'A'
                     WHERE CODIPRODUCTE = v_oldversio;
               
          -- HO REGISTREM A HISTORIC DE VERSIONS
              INSERT INTO VERSIONS_HIST
                  VALUES (null, v_oldversio, sysdate, null);  
                  
              UPDATE VERSIONS_HIST
                   SET DATAFI = sysdate
                      WHERE IDPRODUCTE = v_id AND DATAFI IS NULL;
          
          -- CANVIEM L'ESTAT A INACTIU AL PRODUCTE DONAT DE BAIXA
              UPDATE PRODUCTE
                  SET ESTATVERSIO='I'
                      WHERE CODIPRODUCTE = v_id;
          END IF;
          
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
 COMMIT;

 EXCEPTION 

       WHEN EX_IDPRODUCTE_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: ELS PARAMETRES ENTRADA CODI APLICACIO, SSOO i VERSIO NO PODEN SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);  

         WHEN NO_DATA_FOUND THEN     -- CONTROL QUE EL PRODUCTE EXISTEIXI A LA TAULA PRODUCTE
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PRODUCTE NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

         WHEN EX_PRODUCTE_INACTIU THEN   -- CONTROL QUE EL PRODUCTE NO TINGUI JA EN ESTAT INACTIU
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: PRODUCTE INACTIU '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
                
        WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
             
  END BAIXA_PRODUCTE;

END PKG_ABM_PRODUCTE;

/
--------------------------------------------------------
--  DDL for Package Body PKG_ABM_VIDEO
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "APPS"."PKG_ABM_VIDEO" AS

  PROCEDURE ALTA_VIDEO (
                          a_idproducte    IN VIDEO.IDPRODUCTE%TYPE,
                          a_ididioma      IN VIDEO.IDIDIOMA%TYPE,
                          a_enllavideo   IN VIDEO.ENLLAVIDEO%TYPE,
                          a_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
  v_prodediment := 'PKG_ABM_ALTA_VIDEO ';	
  v_param_entrada :=   TRIM(SUBSTR('PRODUCTE NUM: '||a_idproducte||
                        ' / IDIOMA: '              ||a_ididioma||
                        ' / ENLLA: '              ||SUBSTR(a_enllavideo, 1, 250),1,1000));
       
        -- INSERTEM EL NOU VIDEO D'UN PRODUCTE EN UN IDIOMA DETERMINAT
          	INSERT INTO VIDEO 
                    VALUES (NULL, a_idproducte,a_ididioma, a_enllavideo);
       
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);

       COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA: PRODUCTE + IDIOMA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
  END ALTA_VIDEO;

  PROCEDURE MODIF_VIDEO (
                          m_idvideo       IN VIDEO.IDVIDEO%TYPE,
                          m_idproducte    IN VIDEO.IDPRODUCTE%TYPE,
                          m_ididioma      IN VIDEO.IDIDIOMA%TYPE,
                          m_enllavideo   IN VIDEO.ENLLAVIDEO%TYPE,
                          m_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN
     v_prodediment :=   'PKG_ABM_MODIF_VIDEO ';	
     v_param_entrada := 'IDENTIF. VIDEO: '  ||m_idvideo||
                        ' / PRODUCTE NUM.: '||m_idproducte||
                        ' / IDIOMA: '       ||m_ididioma||
                        ' / ENLLA: '       ||SUBSTR(m_enllavideo, 1, 250);

     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_idvideo NO SIGUI NULL
            IF (m_idvideo IS NULL ) THEN
                  RAISE EX_IDVIDEO_NUL;
            END IF;
            
     -- VERIFIQUEM QUE EL VIDEO EXISTEIXI A LA TAULA VIDEO
          SELECT IDVIDEO  INTO   v_id
                 FROM   VIDEO
                       WHERE  IDVIDEO = m_idvideo;
      
     -- MODIFIQUEM L'ENLLA DEL VIDEO D'UN PRODUCTE EN UN IDIOMA DETERMINAT
          UPDATE  VIDEO
              SET 
                  IDPRODUCTE =  m_idproducte,
                  IDIDIOMA =    m_ididioma,
                  ENLLAVIDEO = m_enllavideo
                       WHERE IDVIDEO = m_idvideo;
          
      -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
COMMIT;
   
EXCEPTION 
          
           WHEN EX_IDVIDEO_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI VIDEO NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

           WHEN NO_DATA_FOUND THEN  -- CONTROL QUE EL VIDEO EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL VIDEO NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

           WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
             -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
          WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA: PRODUCTE + IDIOMA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

  END MODIF_VIDEO;

  PROCEDURE BAIXA_VIDEO (
                           b_idvideo       IN VIDEO.IDVIDEO%TYPE,
                           b_rsp           OUT NOCOPY LOGS.paramOut%TYPE) AS
  BEGIN

    v_prodediment :=    'PKG_ABM_BAIXA_VIDEO ';	
    v_param_entrada :=  'CODI VIDEO: '||b_idvideo; 

     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA b_idvideo NO SIGUI NULL
            IF (b_idvideo IS NULL ) THEN
                  RAISE EX_IDVIDEO_NUL;
            END IF;
            
     -- VERIFIQUEM QUE EL VIDEO EXISTEIXI A LA TAULA VIDEO
          SELECT IDVIDEO  INTO   v_id
                 FROM   VIDEO
                        WHERE  IDVIDEO = b_idvideo;
     
      -- ESBORREM L'ENLLA DEL VIDEO D'UN PRODUCTE EN UN IDIOMA DETERMINAT
          DELETE 
                FROM VIDEO 
                        WHERE  IDVIDEO = b_idvideo;
      
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
            
   COMMIT;

   EXCEPTION 
   
        WHEN EX_IDVIDEO_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI VIDEO NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

         WHEN NO_DATA_FOUND THEN     -- CONTROL QUE EL VIDEO EXISTEIXI A LA TAULA VIDEO
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL VIDEO NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
                
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
  END BAIXA_VIDEO;

END PKG_ABM_VIDEO;

/
--------------------------------------------------------
--  DDL for Procedure ALTA_DESCARREGA
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."ALTA_DESCARREGA" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: ALTA_DESCARREGA.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_DESCARREGA
   DESCRIPCI:          AFEGEIX LES DADES D'UNA DESCARREGA A LA TAULA DESCARREGA.
   PARAMETRES ENTRADA:  a_idproducte, a_iddispositiu,  a_idpagament, a_datadesc
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
                         ERROR: EL PRODUCTE EXISTEIXI I ESTIGUI ACTIU PER FER LA DESCARREGA
                         ERROR: EL PRODUCTE NO TE CAP DESENVOLUPADOR ASSIGNAT   
                         ERROR: EL DISPOSITIU EXISTEIXI I ESTIGUI ACTIU PER FER LA DESCARREGA
                         ERROR: L'USUARI EXISTEIXI I ESTIGUI ACTIU PER FER LA DESCARREGA
                         ERROR: L'OPERADOR DE L'USUARI EXISTEIX I ESTIGUI ACTIU PER FER LA DESCARREGA
                         ERROR: EL PREU NO EXISTEIX
                         ERROR: CLAU FORANA INCORRECTA
   EXEMPLE DE CRIDA:    ALTA_DESCARREGA (254, 54, 1, '01/02/2012', RSP);
***********************************************************************************/
                        a_idproducte        IN DESCARREGA.IDPRODUCTE%TYPE, 
                        a_iddispositiu      IN DESCARREGA.IDDISPOSITIU%TYPE, 
                        a_idpagament        IN DESCARREGA.IDPAGAMENT%TYPE, 
                        a_datadesc          IN DESCARREGA.DATADESC%TYPE, 
                        a_rsp               OUT NOCOPY LOGS.paramOut%TYPE )

IS 

/* VARIABLES LOCALS */
    v_prodediment         LOGS.PROCEDIMENT%TYPE;
    v_param_entrada       LOGS.PARAMIN%TYPE; 
    v_error               number;
    v_desc                VARCHAR2(256);
    v_OK                  number;
    v_idDesenv            number;
    v_idUsuari            DISPOSITIU.IDUSUARI%TYPE;
    v_idOperador          USUARIS.IDOPERADOR%TYPE;
    v_idPaisRegistre      USUARIS.IDPAISREGISTRE%TYPE;
    v_preupagat           number;
    v_datadesc            date;
     
    EX_INS_NUL            EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL             EXCEPTION;   -- no existeix la clau forana
    EX_DESENV_NUL         EXCEPTION;   -- el producte a descarregar no te cap DESENVOLUPADOR 
    EX_PRODUCTE_INACTIU   EXCEPTION;   -- el producte a descarregar esta INACTIU
    EX_DISPOSITIU_INACTIU EXCEPTION;   -- el dispositiu per la descarrega esta INACTIU
    EX_USUARI_INACTIU     EXCEPTION;   -- l'usuari que fa la descarrega esta INACTIU
    EX_OPERADOR_INACTIU   EXCEPTION;   -- l'operador que fa la descarrega esta INACTIU
    EX_PREU_NOK           EXCEPTION;   -- el preu del producte no existeix
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

BEGIN
     v_prodediment :=   'ALTA_DESCARREGA ';	
     v_param_entrada := 'CODI PRODUCTE: '         ||a_idproducte||
                        ' / CODI DISPOSITU: '     ||a_iddispositiu||
                        ' / DATA DESCARREGA: '    ||a_datadesc||
                        ' / CODI FORMA PAGO: '    ||a_idpagament;

    -- VERIFICA QUE EL PRODUCTE EXISTEIX I ESTIGUI ACTIU PER FER LA DESCARREGA.
        SELECT COUNT (1)  INTO v_OK
              FROM PRODUCTE
                  WHERE CODIPRODUCTE = a_idproducte AND ESTATVERSIO = 'A';
        IF (v_OK = 0) THEN
            RAISE EX_PRODUCTE_INACTIU;
        END IF;      
        
    -- VERIFICA QUE EL PRODUCTE TINGUI ASSIGNAT UN DESENVOLUPADOR
        SELECT COUNT(IDDESEV) INTO v_idDesenv 
              FROM AUTOR
                  WHERE IDPROD = a_idproducte;
        IF (v_idDesenv = 0) THEN
            RAISE EX_DESENV_NUL;
        END IF; 
        
    -- VERIFICA QUE EL DISPOSITIU EXISTEIXI I ESTIGUI ACTIU PER FER LA DESCARREGA.
        SELECT COUNT (1)  INTO v_OK
              FROM DISPOSITIU
                  WHERE IDDISPOSITIU = a_iddispositiu AND ESTAT = 'A';
        IF (v_OK = 0) THEN
            RAISE EX_DISPOSITIU_INACTIU;
        END IF;      

    -- CERCA L'USUARI DEL DISPOSITIU I VERIFICA QUE EXISTEIX I ESTIGUI ACTIU PER FER LA DESCARREGA.
        SELECT IDUSUARI  INTO v_idUsuari
              FROM DISPOSITIU 
                  WHERE IDDISPOSITIU = a_iddispositiu;
                  
       SELECT COUNT (1)  INTO v_OK
              FROM USUARIS
                  WHERE IDUSUARI = v_idUsuari AND ESTAT = 'A';
        IF (v_OK = 0) THEN
            RAISE EX_USUARI_INACTIU;
        END IF;      
        
    -- VERIFICA QUE L'OPERADOR DE L'USUARI EXISTEIX I ESTIGUI ACTIU PER FER LA DESCARREGA.
    -- CERCA EL PAIS DE REGISTRE DE L'USUARI EN EL MOMENT DE LA DESCARREGA
        SELECT IDOPERADOR, IDPAISREGISTRE INTO v_idOperador, v_idPaisRegistre
              FROM USUARIS
                  WHERE IDUSUARI = v_idUsuari;
        SELECT COUNT (1)  INTO v_OK
              FROM OPERADORS
                  WHERE IDOPERADOR = v_idOperador AND ESTAT = 'A';
        IF (v_OK = 0) THEN
            RAISE EX_OPERADOR_INACTIU;
        END IF;      
  
    -- VERIFIQUEM SI EL PREU EXISTEIX
       SELECT COUNT (1)  INTO v_OK
              FROM PREUS
                  WHERE IDPRODUCTE = a_idProducte AND IDPAIS = v_idPaisRegistre;
        IF (v_OK = 0) THEN
            RAISE EX_PREU_NOK;
        END IF;
 
     -- OBTENIR EL PREU A PAGAR       
        v_preupagat := GET_PREU_APAGAR(a_idproducte,a_iddispositiu,v_idPaisRegistre);
        
   -- INSERTEM LA NOVA DESCARREGA

          	INSERT INTO DESCARREGA 
                VALUES (NULL, a_datadesc, a_idproducte, a_iddispositiu, v_idUsuari,  
                        v_idOperador, v_idPaisRegistre, v_preupagat, NULL, a_idpagament);
                   --     returning datadesc into v_datadesc;
         
   -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
   -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
            
       COMMIT;
     
   -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
       
       ACT_ESTADISTIQUES(extract(year from a_datadesc),v_idPaisRegistre,a_rsp);
      COMMIT;    

EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
          WHEN NO_DATA_FOUND THEN  -- CONTROL DADES CORRECTES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: DADA NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN EX_PRODUCTE_INACTIU THEN  -- CONTROL PRODUCTE INACTIU 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: PRODUCTE INCORRECTE O INACTIU '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
 
          WHEN EX_DISPOSITIU_INACTIU THEN  -- CONTROL DISPOSITIU INACTIU 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: DISPOSITIU INACTIU '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
                
          WHEN EX_USUARI_INACTIU THEN  -- CONTROL USUARI DONAT DE BAIXA 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: USUARI DONAT DE BAIXA '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);      

          WHEN EX_OPERADOR_INACTIU THEN  -- CONTROL OPERADOR DONAT DE BAIXA 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: OPERADOR DONAT DE BAIXA '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);     
               
          WHEN EX_PREU_NOK  THEN  -- CONTROL PREU PRODUCTE
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: NO EXISTEIX PREU PER AQUEST PRODUCTE '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);     
        
          WHEN EX_DESENV_NUL THEN  -- CONTROL: el producte a descarregar no te cap DESENVOLUPADOR 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: PRODUCTE SENSE DESENVOLUPADOR '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);       
               
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

END ALTA_DESCARREGA;

/
--------------------------------------------------------
--  DDL for Procedure ALTA_DESENVOL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."ALTA_DESENVOL" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: ALTA_DESENVOLUPADOR.sql
************************************************************************************/
/************************************************************************************
   PROCEDIMENT:         ALTA_DESENVOLUPADOR
   DESCRIPCI:          AFEGEIX LES DADES D'UN NOU DESENVOLUPADOR A LA TAULA DESENVOLUPADOR               
                        ACTUALITZA LES DADES ESTADISTICA DEL MODUL_EST_345
   PARAMETRES ENTRADA:  a_nomdesen, a_codiident, a_adreca, a_idlocalitat, a_codipostal,
                        a_numtelefon, a_paginaweb, a_email, a_idpaisregistre, a_idrepresentant
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: CLAU FORANA INCORRECTA
                         ERROR: PARAMETRE ENTRADA NO POT SER NULL
                         ERROR: CLAU UNICA: CODI IDENTIFICATIU, NO POT HAVER-HI DUPLICATS
                         ERROR: NUMERO TELEFON HA DE TENIR 9 DIGITS
   EXEMPLE DE CRIDA:    ALTA_DESENVOLUPADOR ('LEVELUP','B25369488','C/ TIVIDAVO,255',9,'28140',
                                 '912365898','LEVELUP.com','LEVELUP@LEVELUP.COM',1,1,RSP);
***********************************************************************************/
                        a_nomdesen       IN DESENVOLUPADOR.NOMDESEN%TYPE, 
                        a_codiident      IN DESENVOLUPADOR.CODIIDENT%TYPE, 
                        a_adreca         IN DESENVOLUPADOR.ADRECA%TYPE, 
                        a_idlocalitat    IN DESENVOLUPADOR.IDLOCALITAT%TYPE, 
                        a_codipostal     IN DESENVOLUPADOR.CODIPOSTAL%TYPE,
                        a_numtelefon     IN DESENVOLUPADOR.NUMTEL%TYPE, 
                        a_paginaweb      IN DESENVOLUPADOR.PAGINAWEB%TYPE, 
                        a_email          IN DESENVOLUPADOR.EMAIL%TYPE,   
                        a_idpaisregistre IN DESENVOLUPADOR.IDPAISREGISTRE%TYPE, 
                        a_idrepresentant IN DESENVOLUPADOR.IDREPRESENTANT%TYPE, 
                        a_rsp            OUT NOCOPY LOGS.paramOut%TYPE )
IS

/* VARIABLES LOCALS */
    v_prodediment     LOGS.PROCEDIMENT%TYPE;
    v_param_entrada   LOGS.PARAMIN%TYPE; 
    v_error           number;
    v_desc            VARCHAR2(256);

    EX_INS_NUL        EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL         EXCEPTION;   -- no existeix la clau forana

    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

BEGIN
     v_prodediment :=   'ALTA_DESENVOLUPADOR ';	
     v_param_entrada := 'NOM: '                  || a_nomdesen||
                        ' / CODI IDENTIFICADOR: '||a_codiident||
  			                ' / ADREA: '            ||a_adreca||
                        ' / LOCALITAT: '         ||a_idlocalitat||
                        ' / CODI_POSTAL: '       ||a_codipostal||
                        ' / TELEFON: '           ||a_numtelefon||
                        ' / PAGINA WEB: '        ||a_paginaweb||      
                        ' / EMAIL: '             ||a_email||                       
                        ' / PAIS REGISTRE: '     ||a_idpaisregistre||
                        ' / REPRESENTANT: '      ||a_idrepresentant;
       
        -- INSERTEM EL NOU DESENVOLUPADOR

          	INSERT INTO DESENVOLUPADOR 
                VALUES (NULL, UPPER(a_nomdesen),UPPER(a_codiident), UPPER(a_adreca), 
                       a_idlocalitat, a_codipostal, a_numtelefon, a_paginaweb, a_email, 
                       a_idpaisregistre, a_idrepresentant, null, null);
         
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
            
COMMIT;

       -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
            MODUL_EST_345(extract(year from SYSDATE),a_rsp);     
COMMIT;        

EXCEPTION 

    WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

    WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

    WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

    WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

END ALTA_DESENVOL;

/
--------------------------------------------------------
--  DDL for Procedure ALTA_LOGS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."ALTA_LOGS" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: ALTA_LOGS.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         ALTA_LOGS
   DESCRIPCI:          EMMAGATZEMA TOTES LES CRIDES A PROCEDIMENTS A LA TAULA LOGS.
   PARAMETRES ENTRADA:  a_nom_procediment, a_param_entrada, a_param_sortida
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   EXEMPLE DE CRIDA:    ALTA_LOGS ('ALTA_USUARI', 'NOM: MERCE / PRIMER_COGNOM: JORQUERA 
                                   / SEGON_COGNOM: TEIXIDO / NIF: 40883899a 
                                   / TELEFON: 606323123 / ADREA: Passeig Ronda, 787 3-1 
                                   / LOCALITAT: 1 / CODI_POSTAL: 25006 / PAIS REGISTRE: 1 
                                   / EMAIL: mjorquera@xxx.com / OPERADOR: 1 / ESTAT: A 
                                   / DATA ESTAT: 16/11/12','RSP: OK');
********************************************************************************************/

                      a_nom_procediment IN logs.procediment%TYPE,
                      a_param_entrada   IN logs.paramin%TYPE,
                      a_param_sortida   IN logs.paramout%TYPE   
)
IS

/* VARIABLES LOCALS */

    v_prodediment   LOGS.PROCEDIMENT%TYPE;
    v_param_entrada LOGS.PARAMIN%TYPE; 
    v_rsp           logs.paramout%TYPE;
    v_error         number;
    v_desc          VARCHAR2(256);

    EX_INS_NUL      EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
   
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN
    v_prodediment :=   'ALTA_LOGS';	
    v_param_entrada:=   TRIM(SUBSTR(' PROCEDIMENT: ' ||a_nom_procediment||
                       ' / PARAMETRE ENTRADA: '      ||a_param_entrada||
                       ' / PARAMETRE SORTIDA: '      ||a_param_sortida,1,1000));

   -- INSERTEM LA NOVA ENTRADA A LA TAULA LOGS
    	INSERT INTO LOGS VALUES (null, null, a_nom_procediment, a_param_entrada ,a_param_sortida);
      
   -- FINALITZA AMB EXIT
      v_rsp := 'OK'; 
      DBMS_OUTPUT.PUT_LINE(v_prodediment|| ': '||v_rsp);

   COMMIT;

   EXCEPTION
      WHEN EX_INS_NUL THEN       -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               v_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               INSERT INTO LOGS VALUES (null, null, v_prodediment, v_param_entrada ,'RSP: '||v_rsp);
               DBMS_OUTPUT.PUT_LINE(v_prodediment|| ': '||v_rsp);
   
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               v_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               INSERT INTO LOGS VALUES (NULL, null, v_prodediment, v_param_entrada , v_rsp);
               DBMS_OUTPUT.PUT_LINE(v_rsp);
              
END ALTA_LOGS;

/
--------------------------------------------------------
--  DDL for Procedure ALTA_USUARI
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."ALTA_USUARI" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: ALTA_USUARI.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         ALTA_USUARI
   DESCRIPCI:          AFEGEIX LES DADES D'UN USUARI A LA TAULA USUARIS.
                        ACTUALITZA LES DADES ESTADISTICA DEL MODUL_EST_345
   PARAMETRES ENTRADA:  a_nomusuari, a_cognom1, a_cognom2, a_nif, a_numtelefon, 
                        a_adreca, a_idlocalitat, a_codipostal,
                        a_idpaisregistre, a_email, a_idoperador, a_dataestat
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: CLAU FORANA INCORRECTA
                        ERROR: PARAMETRE ENTRADA NO POT SER NULL
                        ERROR: ESTAT NOMS POT SER A (ALTA)
                        ERROR: PAIS DE REGISTRE OPERADOR/USUARI INCORRECTE
                        ERROR: CLAU UNICA: NIF, NO POT HAVER-HI DUPLICATS
                        ERROR: CLAU UNICA: NUMERO TELEFON, NO POT HAVER-HI DUPLICATS                        
                        ERROR: NUMERO TELEFON HA DE TENIR 9 DIGITS                        
   EXEMPLE DE CRIDA:    ALTA_USUARI ('Joan', 'Castello', 'Castello', '43536988V',
                                     '698784752', 'C/ MAJOR, 5', 2, '25140',
                                     1,'joan_castello@gmail.com', 1,'1/01/2011',RSP);
***********************************************************************************/
                        a_nomusuari      IN USUARIS.NOMUSUARI%TYPE, 
                        a_cognom1        IN USUARIS.COGNOM1%TYPE, 
                        a_cognom2        IN USUARIS.COGNOM2%TYPE,
                        a_nif            IN USUARIS.NIF%TYPE,
                        a_numtelefon     IN USUARIS.NUMTELEFON%TYPE, 
                        a_adreca         IN USUARIS.ADRECA%TYPE, 
                        a_idlocalitat    IN USUARIS.IDLOCALITAT%TYPE, 
                        a_codipostal     IN USUARIS.CODIPOSTAL %TYPE,
                        a_idpaisregistre IN USUARIS.IDPAISREGISTRE%TYPE, 
                        a_email          IN USUARIS.EMAIL%TYPE, 
                        a_idoperador     IN USUARIS.IDOPERADOR%TYPE, 
                        a_dataestat      IN USUARIS.DATAESTAT%TYPE,  
                        a_rsp            OUT NOCOPY LOGS.paramOut%TYPE )
IS

/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_paisOperador      OPERADORS.IDPAISREGISTRE%TYPE;

    EX_INS_NUL          EXCEPTION;    -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL           EXCEPTION;    -- no existeix la clau forana
    EX_PR_NOK           EXCEPTION;    -- el pais de registre de l'operador incorrecte 
    EX_TELEFON_NOK      EXCEPTION;    -- el numero de telefon ha de tenir 9 digits
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

BEGIN
     v_prodediment :=   'ALTA_USUARI ';	
     v_param_entrada := 'NOM: '               ||a_nomusuari||
                        ' / PRIMER_COGNOM: '  ||a_cognom1||
                        ' / SEGON_COGNOM: '   ||a_cognom2||
                        ' / NIF: '            ||a_nif||
                        ' / TELEFON: '        ||a_numtelefon||
				                ' / ADREA: '         ||a_adreca||
                        ' / LOCALITAT: '      ||a_idlocalitat||
                        ' / CODI_POSTAL: '    ||a_codipostal||
                        ' / PAIS REGISTRE: '  ||a_idpaisregistre||
                        ' / EMAIL: '          ||a_email||
                        ' / OPERADOR: '       ||a_idoperador||
                        ' / DATA ESTAT: '     ||a_dataestat;
 
     -- VERIFIQUEM QUE EL PAIS DE REGISTRE DE L'OPERADOR ES EL MATEIX QUE EL DE L'USUARI
            SELECT IDPAISREGISTRE INTO v_paisOperador
                FROM OPERADORS
                  WHERE IDOPERADOR = a_idoperador;
            IF (a_idpaisregistre <> v_paisOperador) THEN
                RAISE EX_PR_NOK;
            END IF;    
            
      -- VERIFIQUEM QUE EL NUMERO DE TELEFON TINGUI 9 DIGITS
            IF (LENGTH(a_numtelefon) <> 9) THEN
                RAISE EX_TELEFON_NOK;
            END IF;
      
     -- INSERTEM EL NOU USUARI

          	INSERT INTO USUARIS 
                VALUES (NULL,UPPER(a_nomusuari), UPPER(a_cognom1), UPPER(a_cognom2), UPPER(a_nif), a_numtelefon, UPPER(a_adreca), 
                        a_idlocalitat, a_codipostal,a_idpaisregistre, a_email, a_idoperador, null, a_dataestat);
         
       -- FINALITZA AMB EXIT
          	a_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||a_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
             
       COMMIT;

       -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
            MODUL_EST_345(extract(year from SYSDATE),a_rsp);
       COMMIT;     
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);
               
          WHEN EX_PR_NOK THEN   -- CONTROL PAIS REGISTRE USUARI = PAIS DE REGISTRE DE L'OPERADOR
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: PAIS DE REGISTRE OPERADOR/USUARI INCORRECTE '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);  
               
         WHEN EX_TELEFON_NOK THEN   -- CONTROL QUE EL NUMERO DE TELEFON TINGUI 9 DIGITS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR: EL NUMERO DE TELEFON HA DE TENIR 9 DIGITS '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);     
          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               a_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||a_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||a_rsp);

END ALTA_USUARI;

/
--------------------------------------------------------
--  DDL for Procedure BAIXA_DESENVOL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."BAIXA_DESENVOL" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: BAIXA_DESENVOL.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         BAIXA_DESENVOL
   DESCRIPCI:          ESBORRA LES DADES D'UN DESENVOLUPADOR DE LA TAULA  
			                  SI NO TE CAP PRODUCTE ASSOCIAT, EN CAS QUE EN TINGUI 
                        ES CANVIAR L'ESTAT A 'B' (BAIXA)
                        ACTUALITZA LES DADES ESTADISTICA DEL MODUL_EST_345
   PARAMETRES ENTRADA:  b_iddesen
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA NO POT SER NULL
                        ERRORS: DESENVOLUPADOR NO EXISTEIX
                        ERRORS: DESENVOLUPADOR EN ESTAT INACTIU
   EXEMPLE DE CRIDA:    BAIXA_DESENVOL(11,RSP);
***********************************************************************************/

                        b_iddesen        IN DESENVOLUPADOR.IDDESENV%TYPE,
                        b_rsp            OUT NOCOPY LOGS.paramOut%TYPE )
      
IS 

/* VARIABLES LOCALS */
    v_id              DESENVOLUPADOR.IDDESENV%TYPE;
    v_prodediment     LOGS.PROCEDIMENT%TYPE;
    v_param_entrada   LOGS.PARAMIN%TYPE; 
    v_estat           DESENVOLUPADOR.ESTAT%TYPE;
    v_Total           number;
    v_error           number;
    v_desc            VARCHAR2(256);

    EX_DESENVOL_INACTIU EXCEPTION;  -- si el DESENVOLUPADOR ja esta en estat de baixa
    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    
BEGIN
    v_prodediment :=   'BAIXA_DESENVOL ';	
    v_param_entrada := 'IDENTIF. DESENVOLUPADOR: '||b_iddesen;

    -- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
          IF (b_iddesen IS NULL ) THEN
              RAISE EX_INS_NUL;
          END IF;
        
    -- VERIFIQUEM QUE EL DESENVOLUPADOR EXISTEIXI A LA TAULA DESENVOLUPADOR
          SELECT IDDESENV  INTO   v_id
                 FROM   DESENVOLUPADOR
                        WHERE  IDDESENV = b_iddesen;
 
    -- VERIFIQUEM QUE EL DESENVOLUPADOR NO ESTIGUI JA EN ESTAT DE BAIXA
          SELECT ESTAT  INTO   v_estat
                 FROM   DESENVOLUPADOR
                        WHERE  IDDESENV = b_iddesen;
          IF (UPPER(v_estat) ='B') THEN
                 RAISE EX_DESENVOL_INACTIU;
          END IF;   
          
     -- VERIFIQUEM QUE EL DESENVOLUPADOR NO TINGUI ASSOCIAT CAP PRODUCTE
          SELECT COUNT(IDDESEV) INTO v_Total
                 FROM AUTOR
                       WHERE IDDESEV = b_iddesen;

          IF (v_Total > 0)  THEN
                -- CANVIEM L'ESTAT A BAIXA
                    UPDATE DESENVOLUPADOR
                        SET ESTAT     = 'B',
                            DATAESTAT =  SYSDATE
                          WHERE IDDESENV = b_iddesen;
          ELSE
                -- ESBORREM LES DADES DEL DESENVOLUPADOR DE LA TAULA DESENVOLUPADOR
                    DELETE FROM DESENVOLUPADOR WHERE IDDESENV = b_iddesen;    
          END IF; 
       
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);   
COMMIT;

      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
            MODUL_EST_345(extract(year from SYSDATE),b_rsp);
COMMIT;    

EXCEPTION 

      WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);


      WHEN NO_DATA_FOUND THEN     -- CONTROL QUE EL DESENVOLUPADOR EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL DESENVOLUPADOR NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
      WHEN EX_DESENVOL_INACTIU THEN   -- CONTROL QUE EL DESENVOLUPADOR NO ESTIGUI JA EN ESTAT DE BAIXA
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: DESENVOLUPADOR NO ESTIGUI JA EN ESTAT DE BAIXA '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
     WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);          

END BAIXA_DESENVOL;

/
--------------------------------------------------------
--  DDL for Procedure BAIXA_USUARI
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."BAIXA_USUARI" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: BAIXA_USUARI.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         BAIXA_USUARI
   DESCRIPCI:          ESBORRA LES DADES D'UN USUARI A LA TAULA USUARIS SI NO TE CAP
                        DESCARREGA ASSOCIADA, EN CAS QUE EN TINGUI ALGUNA 
                        ES CANVIAR L'ESTAT A 'B' (BAIXA)
                        ACTUALITZA LES DADES ESTADISTICA DEL MODUL_EST_345
   PARAMETRES ENTRADA:  b_idusuari
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA NO POT SER NULL 
                        ERROR: USUARI NO EXISTEIX
                        ERROR: USUARI EN ESTAT INACTIU
   EXEMPLE DE CRIDA:    BAIXA_USUARI (11,RSP);
***********************************************************************************/
                        b_idusuari       IN USUARIS.idusuari%TYPE,
                        b_rsp            OUT NOCOPY LOGS.paramOut%TYPE )
IS
/* VARIABLES LOCALS */
    v_idUsuari          USUARIS.IDUSUARI%TYPE;
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_TotalDescar       number;
    v_estat             USUARIS.ESTAT%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);

    EX_IDUSU_NUL        EXCEPTION;  -- el codi de l'usuari no es pot ser null
    EX_USUARI_INACTIU   EXCEPTION;  -- si l'usuari ja esta en estat de baixa
    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN

    v_prodediment :=   'BAIXA_USUARI ';	
    v_param_entrada := 'IDENTIFICADOR USUARI: '||b_idusuari;
 
 -- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (b_idusuari IS NULL ) THEN
            RAISE EX_IDUSU_NUL;
        END IF;
 
     -- VERIFIQUEM QUE L'USUARI EXISTEIXI A LA TAULA USUARIS
          SELECT IDUSUARI  INTO   v_idUsuari
                 FROM   USUARIS
                        WHERE  IDUSUARI = b_idusuari;

    -- VERIFIQUEM QUE L'USUARI NO ESTIGUI JA EN ESTAT DE BAIXA
          SELECT ESTAT  INTO   v_estat
                 FROM   USUARIS
                        WHERE  IDUSUARI = b_idusuari;
          IF (UPPER(v_estat) ='B') THEN
                 RAISE EX_USUARI_INACTIU;
          END IF;

      -- VERIFIQUEM QUE L'USUARI NO TINGUI ASSOCIADA CAP DESCARREGA
          SELECT COUNT(IDUSUARI) INTO v_TotalDescar
                 FROM DESCARREGA
                        WHERE IDUSUARI = b_idusuari;
        
          IF (v_TotalDescar > 0) THEN
                -- CANVIEM L'ESTAT A BAIXA
                    UPDATE USUARIS
                        SET ESTAT     = 'B',
                            DATAESTAT =  SYSDATE
                          WHERE IDUSUARI = b_idusuari; 
          ELSE
               -- ESBORREM LES DADES DE L'USUARI DE LA TAULA USUARIS
                    DELETE FROM USUARIS WHERE IDUSUARI = b_idusuari;    
          END IF;
 
      -- FINALITZA AMB EXIT
          	b_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||b_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
   
   COMMIT;
   
      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
            MODUL_EST_345(extract(year from SYSDATE),b_rsp);
   COMMIT;        
   EXCEPTION 

      WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
      
      WHEN EX_IDUSU_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI USUARI NO POT SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);

      WHEN NO_DATA_FOUND THEN     -- CONTROL QUE L'USUARI EXISTEIXI A LA TAULA USUARIS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: USUARI NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
      WHEN EX_USUARI_INACTIU THEN   -- CONTROL QUE L'USUARI NO TINGUI JA EN ESTAT INACTIU
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR: USUARI INACTIU '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
               
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               b_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||b_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||b_rsp);
  
END BAIXA_USUARI;

/
--------------------------------------------------------
--  DDL for Procedure MODIF_DESENVOL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."MODIF_DESENVOL" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: MODIF_DESENVOL.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         MODIF_DESENVOL
   DESCRIPCI:          MODIFICA LES DADES D'UN DESENVOLUPADOR A LA TAULA USUARIS.
                        SI UN DESENVOLUPADOR CANVIA DE NOM S'ACTUALITZA A L'ESTADISTICA 
                        MODUL_EST_345
   PARAMETRES ENTRADA:  m_iddesen, m_nomdesen, m_codiident, m_adreca, m_idlocalitat, m_codipostal,
                        m_numtelefon, m_paginaweb, m_email, m_idpaisregistre, m_idrepresentant
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI DESENVOLUPADOR NO POT SER NULL
                        ERROR: DESENVOLUPADOR NO EXISTEIX
                        ERROR: CLAU FORANA INCORRECTA
                        ERROR: VALOR NO POT SER NULL
                        ERROR: ERROR: ESTAT NOMS POT SER A (ALTA)
                        ERROR: CLAU UNICA: CODI IDENTIFICATIU, NO POT HAVER-HI DUPLICATS
                        ERROR: NUMERO TELEFON HA DE TENIR 9 DIGITS                        
   EXEMPLE DE CRIDA:    MODIF_DESENVOL (255,'LEVELUP','B25369488','C/ TIVIDAVO, 255',9,'28140',
                        '912365898','LEVELUP.com','LEVELUP@LEVELUP.COM',1,1,'A',RSP);
***********************************************************************************/
                        m_iddesen        IN DESENVOLUPADOR.IDDESENV%TYPE,
                        m_nomdesen       IN DESENVOLUPADOR.NOMDESEN%TYPE, 
                        m_codiident      IN DESENVOLUPADOR.CODIIDENT%TYPE, 
                        m_adreca         IN DESENVOLUPADOR.ADRECA%TYPE, 
                        m_idlocalitat    IN DESENVOLUPADOR.IDLOCALITAT%TYPE, 
                        m_codipostal     IN DESENVOLUPADOR.CODIPOSTAL %TYPE,
                        m_numtelefon     IN DESENVOLUPADOR.NUMTEL%TYPE, 
                        m_paginaweb      IN DESENVOLUPADOR.PAGINAWEB%TYPE, 
                        m_email          IN DESENVOLUPADOR.EMAIL%TYPE,   
                        m_idpaisregistre IN DESENVOLUPADOR.IDPAISREGISTRE%TYPE, 
                        m_idrepresentant IN DESENVOLUPADOR.IDREPRESENTANT%TYPE, 
                        m_estat          IN DESENVOLUPADOR.ESTAT%TYPE, 
                        m_rsp            OUT NOCOPY LOGS.paramOut%TYPE )

IS

/* VARIABLES LOCALS */
    v_id              DESENVOLUPADOR.IDDESENV%TYPE;
    v_prodediment     LOGS.PROCEDIMENT%TYPE;
    v_param_entrada   LOGS.PARAMIN%TYPE; 
    v_error           number;
    v_desc            VARCHAR2(256);
    v_NOM_old         DESENVOLUPADOR.NOMDESEN%TYPE;
    
    EX_IDDESEN_NUL    EXCEPTION;   -- el codi del desenvolupador no es pot ser null
    EX_INS_NUL        EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL         EXCEPTION;   -- no existeix la clau forana
    EX_EST_NOK        EXCEPTION;   -- l'estat noms pot ser 'A' (ALTA)
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
    
BEGIN
     v_prodediment := 'MODIF_DESENVOLUPADOR';	
     v_param_entrada := 'IDENTIF. DESENVOLUPADOR: ' ||m_iddesen||
                        ' / NOM: '                  ||m_nomdesen||
                        ' / CODI IDENIFICADOR: '    ||m_codiident||
  			                ' / ADREA: '               ||m_adreca||
                        ' / LOCALITAT: '            ||m_idlocalitat||
                        ' / CODI_POSTAL: '          ||m_codipostal||
                        ' / TELEFON: '              ||m_numtelefon||
                        ' / PAGINA WEB: '           ||m_paginaweb||      
                        ' / EMAIL: '                ||m_email||                       
                        ' / PAIS REGISTRE: '        ||m_idpaisregistre||
                        ' / REPRESENTANT: '         ||m_idrepresentant||
                        ' / ESTAT: '                ||m_estat;
    
    -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_iddesen NO SIGUI NULL
              IF (m_iddesen IS NULL ) THEN
                  RAISE EX_IDDESEN_NUL;
              END IF;    
    
    -- VERIFIQUEM QUE EL DESENVOLUPADOR EXISTEIXI A LA TAULA DESENVOLUPADOR
          SELECT IDDESENV  INTO   v_id
                 FROM   DESENVOLUPADOR
                        WHERE  IDDESENV = m_iddesen;
                  
    -- VERIFIQUEM QUE m_estat = 'A', en cas que sigui 'B' DONAR ERROR, PER DONAR DE BAIXA
    -- UN DESENVOLUPADOR S'HA D'ANAR PER BAIXA_DESENVOL
            IF (UPPER(m_estat) <> 'A') THEN
                RAISE EX_EST_NOK;
            END IF;

    -- VERIFIQUEM SI EL DESENVOLUPADOR HA CANVIAT DE NOM (per actualitzar les estadistiques)
          SELECT NOMDESEN INTO v_NOM_old
                 FROM    DESENVOLUPADOR
                        WHERE  IDDESENV = m_iddesen;      
                        
    -- MODIFIQUEM LES DADES DEL DESENVOLUPADOR
          UPDATE DESENVOLUPADOR
                 SET  NOMDESEN =        UPPER(m_nomdesen),
                      CODIIDENT =       m_codiident, 
                      ADRECA =          UPPER(m_adreca),
                      IDLOCALITAT =     m_idlocalitat, 
                      CODIPOSTAL =      m_codipostal, 
                      NUMTEL=           m_numtelefon, 
                      PAGINAWEB=        m_paginaweb,
                      EMAIL =           m_email, 
                      IDPAISREGISTRE =  m_idpaisregistre,
                      IDREPRESENTANT =  m_idrepresentant,
                      ESTAT =           UPPER(m_estat),
                      DATAESTAT =       SYSDATE
                   WHERE IDDESENV = m_iddesen;
          
       -- FINALITZA AMB EXIT
          	m_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
            
 COMMIT;
 
      -- SI EL DESENVOLUPADOR HA CANVIAT DE NOM
      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
      
            IF (UPPER(v_NOM_old) <> UPPER(m_nomdesen)) THEN
                MODUL_EST_345(extract(year from SYSDATE),m_rsp);   
            END IF;
 COMMIT;                
 EXCEPTION 
 
      WHEN EX_IDDESEN_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI DESENVOLUPADOR NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
      WHEN NO_DATA_FOUND THEN  -- CONTROL QUE EL DESENVOLUPADOR EXISTEIXI 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL DESENVOLUPADOR NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
      WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

      WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

       WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
        
       WHEN EX_EST_NOK THEN   -- CONTROL L'ESTAT NOMS POT SER 'A'
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: ESTAT NOMS POT SER A (ALTA) '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);                  

       WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);          
          
END MODIF_DESENVOL;

/
--------------------------------------------------------
--  DDL for Procedure MODIF_USUARI
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."MODIF_USUARI" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: MODIF_USUARI.sql
************************************************************************************/

/************************************************************************************
   PROCEDIMENT:         MODIF_USUARI
   DESCRIPCI:          MODIFICA LES DADES D'UN USUARI A LA TAULA USUARIS.
                        SI UN USUARI INACTIU CANVIA D'ESTAT D'ALTA S'ACTUALITZA A 
                        L'ESTADISTICA MODUL_EST_345 (usuaris actius)
                        SI UN USUARI CANVIA DE PAIS DE REGISTRE, AIXO NO AFECTA A LES 
                        ESTADISTIQUES
   PARAMETRES ENTRADA:  m_idusuari, m_nomusuari, m_cognom1, m_cognom2, m_nif, m_numtelefon, m_adreca, 
                        m_idlocalitat, m_codipostal, m_idpaisregistre, m_email, m_idoperador,m_estat
   PARAMETRES SORTIDA:  RSP
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA CODI USUARI NO POT SER NULL
                        ERROR: USUARI NO EXISTEIX
                        ERROR: CLAU FORANA INCORRECTA
                        ERROR: VALOR NO POT SER NULL
                        ERROR: ERROR: ESTAT NOMS POT SER A (ALTA)
                        ERROR: PAIS DE REGISTRE OPERADOR/USUARI INCORRECTE
                        ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS
                        ERROR: NUMERO TELEFON HA DE TENIR 9 DIGITS
   EXEMPLE DE CRIDA:    MODIF_USUARI (23,'Joan', 'Castello', 'Castello', '43536988V','698784752', 'C/ MAJOR, 5', 2, '25140',
                                 1,'joan_castello@gmail.com', 1,'A', RSP);
***********************************************************************************/
                        m_idusuari       IN USUARIS.idusuari%TYPE,
                        m_nomusuari      IN USUARIS.NOMUSUARI%TYPE, 
                        m_cognom1        IN USUARIS.COGNOM1%TYPE, 
                        m_cognom2        IN USUARIS.COGNOM2%TYPE,
                        m_nif            IN USUARIS.NIF%TYPE,
                        m_numtelefon     IN USUARIS.NUMTELEFON%TYPE, 
                        m_adreca         IN USUARIS.ADRECA%TYPE, 
                        m_idlocalitat    IN USUARIS.IDLOCALITAT%TYPE, 
                        m_codipostal     IN USUARIS.CODIPOSTAL %TYPE,
                        m_idpaisregistre IN USUARIS.IDPAISREGISTRE%TYPE, 
                        m_email          IN USUARIS.EMAIL%TYPE, 
                        m_idoperador     IN USUARIS.IDOPERADOR%TYPE, 
                        m_estat          IN USUARIS.ESTAT%TYPE, 
                        m_rsp            OUT NOCOPY LOGS.paramOut%TYPE )
IS

/* VARIABLES LOCALS */
    v_idUsuari          USUARIS.IDUSUARI%TYPE;
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_paisOperador      OPERADORS.IDPAISREGISTRE%TYPE;
    v_Estat_old         USUARIS.ESTAT%TYPE;
    
    EX_IDUSU_NUL        EXCEPTION;   -- el codi de l'usuari no es pot ser null
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL           EXCEPTION;   -- no existeix la clau forana
    EX_PR_NOK           EXCEPTION;   -- el pais de registre de l'operador incorrecte 
    EX_EST_NOK          EXCEPTION;   -- l'estat noms pot ser 'A' (ALTA)
    EX_TELEFON_NOK      EXCEPTION;   -- el numero de telefon ha de tenir 9 digits
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

BEGIN

     v_prodediment :=   'MODIF_USUARI ';	
     v_param_entrada := 'IDENTIFICADOR USUARI: '||m_idusuari||
                        ' / NOM: '              ||m_nomusuari||
                        ' / PRIMER_COGNOM: '    ||m_cognom1||
                        ' / SEGON_COGNOM: '     ||m_cognom2||
                        ' / NIF: '              ||m_nif||
                        ' / TELEFON: '          ||m_numtelefon||
				                ' / ADREA: '           ||m_adreca||
                        ' / LOCALITAT: '        ||m_idlocalitat||
                        ' / CODI_POSTAL: '      ||m_codipostal||
                        ' / PAIS REGISTRE: '    ||m_idpaisregistre||
                        ' / EMAIL: '            ||m_email||
                        ' / OPERADOR: '         ||m_idoperador||
                        ' / ESTAT: '            ||m_estat;
 
     
     -- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA m_idusuari NO SIGUI NULL
              IF (m_idusuari IS NULL ) THEN
                  RAISE EX_IDUSU_NUL;
              END IF;
          
     -- VERIFIQUEM QUE L'USUARI EXISTEIXI A LA TAULA USUARIS
              SELECT IDUSUARI  INTO   v_idUsuari
                     FROM   USUARIS
                            WHERE  IDUSUARI = m_idusuari;
                      
     -- VERIFIQUEM QUE EL PAIS DE REGISTRE DE L'OPERADOR ES EL MATEIX QUE EL DE L'USUARI
              SELECT IDPAISREGISTRE INTO v_paisOperador
                  FROM OPERADORS
                    WHERE IDOPERADOR = m_idoperador;
              IF (m_idpaisregistre <> v_paisOperador) THEN
                  RAISE EX_PR_NOK;
              END IF;                            
     
     -- VERIFIQUEM QUE m_estat = 'A', en cas que sigui 'B' DONAR ERROR, PER DONAR DE BAIXA
     -- UN USUARI S'HA D'ANAR PER BAIXA_USUARI
              IF (UPPER(m_estat) <> 'A') THEN
                  RAISE EX_EST_NOK;
              END IF;

     -- VERIFIQUEM QUE EL NUMERO DE TELEFON TINGUI 9 DIGITS
              IF (LENGTH(m_numtelefon) <> 9) THEN
                  RAISE EX_TELEFON_NOK;
              END IF;
      
     -- VERIFIQUEM SI L'USUARI HA CANVIAT D'ESTAT (per actualitzar les estadistiques)
              SELECT ESTAT INTO v_Estat_old
                     FROM    USUARIS
                            WHERE  IDUSUARI = m_idusuari;   
                            
     -- MODIFIQUEM LES DADES DE L'USUARI 
              UPDATE USUARIS
                  SET       NOMUSUARI =       UPPER(m_nomusuari),
                            COGNOM1 =         UPPER(m_cognom1), 
                            COGNOM2 =         UPPER(m_cognom2), 
                            NIF =             UPPER(m_nif), 
                            NUMTELEFON=       m_numtelefon, 
                            ADRECA =          UPPER(m_adreca),
                            IDLOCALITAT =     m_idlocalitat, 
                            CODIPOSTAL =      m_codipostal, 
                            IDPAISREGISTRE =  m_idpaisregistre,
                            EMAIL =           m_email, 
                            IDOPERADOR =      m_idoperador,
                            ESTAT =           UPPER(m_estat),
                            DATAESTAT =       SYSDATE
                  WHERE IDUSUARI = m_idusuari;
              
      -- FINALITZA AMB EXIT
                  m_rsp := 'OK'; 
                  DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||m_rsp);
            
      -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
                  ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
   COMMIT;
   
      -- ACTUALITZEM LES DADES A LES ESTADISTIQUES
      -- SI L'USUARI HA CANVIAT D'ESTAT INACTIU A ACTIU
                  IF (UPPER(v_Estat_old) <> UPPER(m_estat)) THEN
                      MODUL_EST_345(extract(year from SYSDATE),m_rsp);
                  END IF;
   COMMIT;    
 EXCEPTION 
 
     WHEN EX_IDUSU_NUL THEN   -- CONTROL PARAMETRE ENTRADA NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA CODI USUARI NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

     WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
     WHEN NO_DATA_FOUND THEN  -- CONTROL QUE L'USUARI EXISTEIXI A LA TAULA USUARIS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: USUARI NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
    WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

     WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);
               
     WHEN EX_PR_NOK THEN   -- CONTROL PAIS REGISTRE USUARI = PAIS DE REGISTRE DE L'OPERADOR
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: PAIS DE REGISTRE OPERADOR/USUARI INCORRECTE '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);                    

     WHEN EX_EST_NOK THEN   -- CONTROL L'ESTAT NOMS POT SER 'A'
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: ESTAT NOMS POT SER A (ALTA) '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);     
               
      WHEN EX_TELEFON_NOK THEN   -- CONTROL QUE EL NUMERO DE TELEFON TINGUI 9 DIGITS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR: EL NUMERO DE TELEFON HA DE TENIR 9 DIGITS '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);    
  
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               m_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||m_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||m_rsp);

END MODIF_USUARI;

/
spool OFF