--------------------------------------------------------
--  DDL for Procedure ACT_ESTADISTIQUES
--------------------------------------------------------
spool CARREGAR_ESTADISTIQUES.log

set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."ACT_ESTADISTIQUES" (
/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: ACT_ESTADISTIQUES.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:          ACT_ESTADISTIQUES
   DESCRIPCI:           ACTUALITZA LES DADES DE TOTES LES ESTADISTIQUES
   PARAMETRES ENTRADA:   e_any, e_pais
   PARAMETRES SORTIDA:   rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: EL PARAMETRE D'ENTRADA NO POT SER NULL
   EXEMPLE DE CRIDA:     ACT_ESTADISTIQUES(2012,1,rsp);
********************************************************************************************/
                        e_anyo  IN number,
                        e_pais  IN number,
                        e_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 

/* VARIABLES LOCALS */

    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               logs.paramout%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);    
    
BEGIN

    v_prodediment :=   'ACT_ESTADISTIQUES ';	
    v_param_entrada := 'ANY : '||e_anyo||
                       ' / PAIS : '||e_pais;

-- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
          IF e_anyo IS NULL OR e_pais IS NULL THEN
              RAISE EX_INS_NUL;
          END IF; 
          
-- ACTUALITZEM LES DADES A LES ESTADISTIQUES
       
       MODUL_EST_12(e_rsp);
       COMMIT;
       MODUL_EST_345(e_anyo,e_rsp);
       COMMIT;
       MODUL_EST_678(e_anyo,e_pais,e_rsp);          
       COMMIT;
 
-- FINALITZA AMB EXIT
       	e_rsp := 'OK'; 
        DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||e_rsp);
             
 -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
  
 COMMIT;
 
 EXCEPTION                             

       WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: ELS PARAMETRES ENTRADA NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
               
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);                  
                       
END ACT_ESTADISTIQUES;

/
--------------------------------------------------------
--  DDL for Procedure MODUL_EST_12
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."MODUL_EST_12" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: MODUL_EST_12.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:          MODUL_EST_12
   DESCRIPCI:           ACTUALITZA LES DADES CORRESPONENTS AL TOTAL DE DESCARREGUES
                         DE LA PLATAFORMA FINS A LA DATA I EL NMERO TOTAL D'EUROS 
                         GENERATS EN DESCARREGUES A LA PLATAFORMA FINS ARA MATEIX.
   PARAMETRES ENTRADA:   SENSE PARAMETRES D'ENTRADA
   PARAMETRES SORTIDA:   rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   EXEMPLE DE CRIDA:     MODUL_EST_12(rsp);
********************************************************************************************/

                   e_rsp  OUT NOCOPY logs.paramout%TYPE   )

AS 

/* VARIABLES LOCALS */

          v_prodediment   LOGS.PROCEDIMENT%TYPE;
          v_param_entrada LOGS.PARAMIN%TYPE; 
          v_rsp           logs.paramout%TYPE;
          v_error         number;
          v_desc          VARCHAR2(256);
          v_Total         number;
          v_Import        number;
          v_Fila          number;
    
BEGIN

    v_prodediment :=  'ESTADISTICA 1 i 2  TOTAL DESCARREGUES/EUROS ';	
    v_param_entrada:= 'SENSE PARAMETRES ENTRADA';
    
-- ESTADISTICA 1 i 2

      -- CALCULAR EL TOTAL DE DESCARREGUES EFECTUADES FINS A LA DATA
            SELECT COUNT(1), SUM(preupagat) INTO v_Total, v_Import  FROM DESCARREGA;
            
      -- ACTUALITZAR EL NUMERO DE DESCARREGUES
      
          -- MIRAR SI EXISTEIX LA FILA DE LA TAULA ESTADISTICA_12
                SELECT COUNT(1) INTO v_Fila FROM ESTADISTICA_12;
                
                -- SI EXISTEIX ACTUALITZEM LES DADES (UPDATE)
                    IF( v_Fila >0) THEN
                        UPDATE estadistica_12 
                          SET TOTALDESCARREGUES = v_Total,
                              TOTALDINERS = ROUND(v_Import,2);
                     ELSE
                -- SI NO EXISTEIX CREEM LA FILA (INSERT)
                        INSERT INTO ESTADISTICA_12
                          VALUES (v_Total,ROUND(v_Import,2));
                     END IF;
     
       -- FINALITZA AMB EXIT
          	e_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||e_rsp);
             
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
             
       COMMIT;
EXCEPTION      
        
      WHEN NO_DATA_FOUND THEN  -- CONTROL DADES CORRECTES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: DADA NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
               
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);

END MODUL_EST_12;

/
--------------------------------------------------------
--  DDL for Procedure MODUL_EST_345
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."MODUL_EST_345" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: MODUL_EST_345.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         MODUL_EST_345
   DESCRIPCI:          ACTUALITZA LES DADES CORRESPONENTS A:
                        - Donat un any concret el nmero mig daplicacions
                          descarregades per un usuari
                        - Donat un any concret, el desenvolupador que tingui el
                          mxim nmero de descrregues (sumant totes les descrregues
                          de totes les seves aplicacions que shagin realitzat aquell any),
                          aix com aquest nmero.  
                        - Donat un any concret, laplicaci que ms diners ha recaudat
                          en descrregues aix com el/s seu/s desenvolupador/s.  
   PARAMETRES ENTRADA:   e_any
   PARAMETRES SORTIDA:   rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: EL PARAMETRE D'ENTRADA NO POT SER NULL
                         ERROR: NO EXISTEIXIN DESCARREGUES D'AQUEST ANY: e_anyo
   EXEMPLE DE CRIDA:     MODUL_EST_345(2012,rsp);
********************************************************************************************/
                        e_anyo  IN number,
                        e_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 

/* VARIABLES LOCALS */

    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               logs.paramout%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_nUsuaris          number;
    v_nAplicacions      number;
    v_mitja             number;
    v_Fila              number;
    v_OK                number;
    v_numDesenv         number;
    v_nomDesenv_E4      desenvolupador.nomdesen%TYPE;
    v_idProducte        descarrega.idproducte%TYPE;
    v_nomAplicacio_E5   VARCHAR2(70);
    v_nomsdesen_E5      VARCHAR2(200);
    
    EX_ANY_NOK          EXCEPTION;   -- l'any introduit com a parametre d'entrada no te descarregues
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN

    v_prodediment :=   'ESTADISTICA 3,4,5 ';	
    v_param_entrada := 'ANY : '||e_anyo;
    
-- VERIFIQUEM QUE EL PARAMETRE D'ENTRADA NO SIGUI NULL
          IF e_anyo IS NULL THEN
              RAISE EX_INS_NUL;
          END IF; 

-- VERIFIQUEM QUE EXISTEIXIN DESCARREGUES D'AQUEST ANY: e_anyo

          SELECT COUNT(1) INTO v_OK
                 FROM DESCARREGA
                        WHERE (extract(year from DATADESC)) = e_anyo;

IF (v_OK = 0) THEN
-- SI NO HI HA DESCARREGUES NO CAL ACTUALITZAR LES ESTADISTIQUES
             -- FINALITZA 
          	    e_rsp := 'NO HI HA DADES PER ACTUALIZAR'; 
                DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||e_rsp);
             
             -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
                ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
                COMMIT;
ELSE             
--  SI HI HA DESCARREGUES S'ACTUALITZEN LES ESTADISTIQUES  

-- ESTADISTICA 3
-- NUMERO MIG D'APLICACIONS DESCARREGADES PER UN USUARI
-- Per l'any actual, # Desc. acumulades durant l'any en el moment de la consulta / # Usuaris actius en el moment de la consulta.
-- Per anys anteriors, # Desc. totals de l'any / # Usuaris actius al final de l'any
    
    -- MIREM EL NUMERO DE DESCARREGUES (APLICACIO+SSOO+VERSIO) ACUMULADES 
    -- DURANT UN ANY DETERMINAT (e_any),SENSE TENIR EN COMPTE LES DESCARREGUES
    -- DELS USUARIS INACTIUS EN AQUELLA DATA
    
      SELECT COUNT(D.IDPRODUCTE) INTO v_nAplicacions
          FROM DESCARREGA D, USUARIS U
              WHERE D.IDUSUARI                    = U.IDUSUARI
                    AND (Extract(YEAR FROM D.DATADESC)  = e_anyo
                    AND U.ESTAT                         = 'A'
                    AND Extract(YEAR FROM U.DATAESTAT) <= e_anyo);

 --   SELECT COUNT(DISTINCT PRODUCTE.IDAPLIC) INTO v_nAplicacions
 --       FROM DESCARREGA, PRODUCTE
 --           WHERE IDPRODUCTE= CODIPRODUCTE 
 --                 AND extract(year from descarrega.datadesc) = e_anyo;

    -- MIREM EL NUMERO D'USUARIS ACTIUS (TANT SI HAN FET DESCARREGUES COM NO)
    
        SELECT COUNT (U.IDUSUARI) INTO v_nUsuaris 
            FROM USUARIS U
              WHERE U.ESTAT ='A' AND Extract(YEAR FROM U.DATAESTAT) <= e_anyo;    

--  NUMERO USUARIS ACTIUS AMB ALGUNA DESCARREGA
/*      SELECT COUNT (distinct D.IDUSUARI) INTO v_nUsuaris 
          FROM DESCARREGA D , DISPOSITIU DI, USUARIS U
              WHERE     D.IDDISPOSITIU = DI.IDDISPOSITIU
                    AND U.IDUSUARI  = DI.IDUSUARI
                    AND U.ESTAT ='A'
                    AND Extract(YEAR FROM U.DATAESTAT) <= e_anyo
                    AND Extract(YEAR FROM D.DATADESC)   = e_anyo;
*/

    -- CALCULEM EL NMERO MIG D'APLICACIONS DESCARREGADES 
           IF  (v_nAplicacions = 0 OR v_nUsuaris = 0) THEN
                  v_mitja := 0;
           ELSE   
                  v_mitja:= ROUND((v_nAplicacions/v_nUsuaris),2);
           END IF;       
    
-- ESTADISTICA 4
-- CERQUEM: el desenvolupador que tingui el mxim nmero de descrregues
-- EN CAS DE PRODUIR-SE UN EMPAT MOSTRAREM NOMES UN DESENVUPADOR
-- DESCARREGUES = APLICACIO + SSOO + VERSIO

        SELECT DISTINCT DESENVOLUPADOR.NOMDESEN, query1.total_desc
            INTO v_nomDesenv_E4, v_nAplicacions
        FROM
            (SELECT AUTOR.IDDESEV,  COUNT(DESCARREGA.IDPRODUCTE) AS total_desc
              FROM DESCARREGA, AUTOR
              WHERE DESCARREGA.IDPRODUCTE = AUTOR.IDPROD
                    AND Extract(YEAR FROM DESCARREGA.DATADESC) = e_anyo
              GROUP BY autor.iddesev
             ) query1,
             (SELECT MAX(query2.total_desc) AS highest_desc
              FROM
                (SELECT AUTOR.IDDESEV,  COUNT(DESCARREGA.IDPRODUCTE) AS total_desc
                  FROM DESCARREGA, AUTOR
                   WHERE DESCARREGA.IDPRODUCTE = AUTOR.IDPROD
                        AND Extract(YEAR FROM DESCARREGA.DATADESC) = e_anyo
                    GROUP BY autor.iddesev
                ) query2
            ) query3,
            AUTOR,
            DESENVOLUPADOR
        WHERE query1.total_desc     = query3.highest_desc
              AND query1.IDDESEV       = AUTOR.IDDESEV
              AND DESENVOLUPADOR.IDDESENV = AUTOR.IDDESEV
              AND ROWNUM =1 ;
    
-- ESTADISTICA 5
-- CALCULEM: laplicaci (APLICACIO + SSOO+ VERSIO) que ms diners ha recaudat en descrregues
--           i el/s seu/s desenvolupador/s
    
         SELECT query1.IDPRODUCTE  INTO v_idProducte
            FROM
                (SELECT DESCARREGA.IDPRODUCTE, SUM(DESCARREGA.PREUPAGAT) AS total_desc
                    FROM DESCARREGA
                    WHERE Extract(YEAR FROM DESCARREGA.DATADESC) = e_anyo
                    GROUP BY DESCARREGA.IDPRODUCTE
                ) query1,
                (SELECT MAX(query2.total_desc) AS highest_desc
                    FROM
                (       SELECT DESCARREGA.IDPRODUCTE, SUM(DESCARREGA.PREUPAGAT) AS total_desc
                          FROM DESCARREGA
                          WHERE Extract(YEAR FROM DESCARREGA.DATADESC) = e_anyo
                          GROUP BY DESCARREGA.IDPRODUCTE
                ) query2
                ) query3
            WHERE query1.total_desc     = query3.highest_desc
                  AND ROWNUM =1;
                  
   -- OBTENIM EL NOM DE L'APLICACIO QUE MES DINERS HA RECAPTAT
          v_nomAplicacio_E5:= GET_NOM_APLICACIO(v_idProducte);
               
               
   -- OBTENIM EL NOM DEL/S DESENVOLUPADOR/S DE L'APLICACIO QUE MES DINERS HA RECAPTAT            
          v_nomsdesen_E5:= GET_NOM_DESENV(v_idProducte);
          
 /*       SELECT DESENVOLUPADOR.NOMDESEN,  APLICACIO.NOMAPLI
          INTO v_nomDesenvApl, v_nomAplicacio
        FROM
            (SELECT DESCARREGA.IDPRODUCTE,
                    SUM(DESCARREGA.PREUPAGAT) AS total_desc
              FROM DESCARREGA
              WHERE Extract(YEAR FROM DESCARREGA.DATADESC) = e_anyo
              GROUP BY DESCARREGA.IDPRODUCTE
            ) query1,
            (SELECT MAX(query2.total_desc) AS highest_desc
              FROM
                (SELECT DESCARREGA.IDPRODUCTE,
                        SUM(DESCARREGA.PREUPAGAT) AS total_desc
                  FROM DESCARREGA
                  WHERE Extract(YEAR FROM DESCARREGA.DATADESC) = e_anyo
                  GROUP BY DESCARREGA.IDPRODUCTE
                ) query2
            ) query3,
          AUTOR,
          DESENVOLUPADOR,
          PRODUCTE,
          APLICACIO
        WHERE query1.total_desc     = query3.highest_desc
            AND query1.IDPRODUCTE       = AUTOR.IDPROD
            AND DESENVOLUPADOR.IDDESENV = AUTOR.IDDESEV
            AND AUTOR.IDPROD            = PRODUCTE.CODIPRODUCTE
            AND APLICACIO.IDAPLICACIO   = PRODUCTE.IDAPLIC
            AND ROWNUM =1;
 */ 
 
    -- ACTUALITZAR TOTES LES DADES A LA TAULA ESTADISTICA_345
      
      -- MIRAR SI EXISTEIX LA FILA DE LA TAULA ESTADISTICA_345
                SELECT COUNT(1) INTO v_Fila 
                FROM ESTADISTICA_345 
                WHERE ANYO = e_anyo;
                
                -- SI EXISTEIX ACTUALITZEM LES DADES (UPDATE)
                    IF( v_Fila >0) THEN
                        UPDATE estadistica_345
                          SET MITJADESCUSU = v_mitja,
                              DESENVMAXDESC = v_nomDesenv_E4,
                              NUMDESCDESEN = v_nAplicacions,
                              APLICMESRECAPTAT = SUBSTR(v_nomAplicacio_E5,1,70),
                              desenvmesrecaptat = v_nomsdesen_E5
                              WHERE ANYO = e_anyo;
                     ELSE
                -- SI NO EXISTEIX CREEM LA FILA (INSERT)
                        INSERT INTO ESTADISTICA_345
                          VALUES (e_anyo,v_mitja,v_nomDesenv_E4,v_nAplicacions,SUBSTR(v_nomAplicacio_E5,1,70),v_nomsdesen_E5);
                     END IF;
     
       -- FINALITZA AMB EXIT
          	e_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||e_rsp);
             
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
           COMMIT;
           
END IF;    

COMMIT;

EXCEPTION      

      WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
        
      WHEN EX_ANY_NOK THEN   -- CONTROL QUE EXISTEIXIN DESCARREGUES D'AQUEST ANY: e_anyo
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: NO EXISTEIXIN DESCARREGUES EN ANY: '||e_anyo||' '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
               
      WHEN NO_DATA_FOUND THEN  -- CONTROL DADES CORRECTES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: DADA NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
               
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);   
               
END MODUL_EST_345;

/
--------------------------------------------------------
--  DDL for Procedure MODUL_EST_678
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."MODUL_EST_678" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: MODUL_EST_678.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         MODUL_EST_678
   DESCRIPCI:          ACTUALITZA LES DADES CORRESPONENTS A:
                        - Donat un any concret i un pas: el nmero dusuaris
                          diferents que han fet com a mnim una descrrega.
                        - Donat un any concret i un pas: el ingressos totals que han
                          generat els usuaris registrats en aquell pas en descrregues
                          daplicacions
                        - Donat un any concret i un pas: el nmero daplicacions
                          diferents descarregades com a mnim una vegada.  
   PARAMETRES ENTRADA:  e_any, e_pais
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:               ERROR: ELS PARAMETRES D'ENTRADA NO PODEN SER NULL
                         ERROR: NO EXISTEIXIN DESCARREGUES D'AQUEST ANY: e_anyo  
                         ERROR: EL PAIS e_pais NO EXISTEIX                         
   EXEMPLE DE CRIDA:     MODUL_EST_678(2012,1,rsp);
********************************************************************************************/
                        e_anyo  IN number,
                        e_pais  IN number,
                        e_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 

/* VARIABLES LOCALS */

    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_rsp               logs.paramout%TYPE;
    v_error             number;
    v_desc              VARCHAR2(256);
    v_Fila              number;
    v_numusumindes      number;
    v_INGRESSOSANYPAIS  number;
    v_NUMAPLICMINDES    number;
    v_OK                number;
    v_OK_PAIS           number;
    
    EX_PAIS_NUL         EXCEPTION;   -- el pais no existeix 
    EX_ANY_NOK          EXCEPTION;   -- l'any introduit com a parametre d'entrada no te descarregues
    EX_PAIS_NOK         EXCEPTION;   -- el pais introduit com a parametre d'entrada no te descarregues
    EX_INS_NUL          EXCEPTION;   -- no es poden insertar valors nulls si estan definits com NOT NULL
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    
BEGIN

    v_prodediment :=   'ESTADISTICA 6,7,8 ';	
    v_param_entrada := 'ANY : '||e_anyo||
                       ' / PAIS : '||e_pais;

-- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
          IF e_anyo IS NULL OR e_pais IS NULL THEN
              RAISE EX_INS_NUL;
          END IF; 

-- VERIFIQUEM QUE EL PAIS EXISTEIXI 
        SELECT COUNT (1)  INTO v_OK
              FROM PAIS
                  WHERE IDPAIS = e_pais;
        IF (v_OK = 0) THEN
            RAISE EX_PAIS_NUL;
        END IF;         

-- VERIFIQUEM QUE EXISTEIXIN DESCARREGUES D'AQUEST ANY: e_anyo
          SELECT COUNT(1) INTO v_OK
                 FROM DESCARREGA
                        WHERE (extract(year from DATADESC)) = e_anyo;
--          IF (v_OK = 0) THEN
--              RAISE EX_ANY_NOK;
--          END IF;  

-- VERIFIQUEM QUE EXISTEIXIN DESCARREGUES D'AQUEST PAIS: e_pais
          SELECT COUNT(1) INTO v_OK_PAIS
                 FROM DESCARREGA
                        WHERE IDPAISREGUSU = e_pais;
--          IF (v_OK = 0) THEN
--              RAISE EX_PAIS_NOK;
--          END IF;     

IF (v_OK = 0 and v_OK_PAIS=0) THEN
-- SI NO HI HA DESCARREGUES NO CAL ACTUALITZAR LES ESTADISTIQUES
             -- FINALITZA 
          	    e_rsp := 'NO HI HA DADES PER ACTUALIZAR'; 
                DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||e_rsp);
             
             -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
                ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
                COMMIT;
ELSE   
--  SI HI HA DESCARREGUES S'ACTUALITZEN LES ESTADISTIQUES  

-- ESTADISTICA 6    
-- Donat un any concret i un pas: el nmero dusuaris
-- diferents que han fet com a mnim una descrrega.

    SELECT COUNT(1) INTO v_numusumindes
      FROM
          (SELECT D.IDUSUARI, COUNT(1) AS total_DESCA
            FROM DESCARREGA D
            WHERE  Extract(YEAR FROM D.DATADESC) = e_anyo and d.idpaisregusu = e_pais
            group by D.IDUSUARI
            having COUNT(1)>=1);


-- ESTADISTICA 7
-- Donat un any concret i un pas: el ingressos totals que han
-- generat els usuaris registrats en aquell pas en descrregues daplicacions
-- DESCARREGUES= APLICACIO + SSOO +VERSIO
    SELECT SUM(total_PREUS) INTO v_INGRESSOSANYPAIS
      FROM
          (SELECT D.IDUSUARI, SUM(D.PREUPAGAT) AS total_PREUS
            FROM DESCARREGA D
            WHERE  Extract(YEAR FROM D.DATADESC) = e_anyo and d.idpaisregusu = e_pais
            group by D.IDUSUARI);


-- ESTADISTICA 8
-- Donat un any concret i un pas: el nmero daplicacions 
-- (sense tenir en compte el sistema operatiu ni la versi) 
-- diferents descarregades com a mnim una vegada.
-- DESCARREGUES= APLICACIO

    SELECT COUNT(1) INTO v_NUMAPLICMINDES
      FROM
          (SELECT DISTINCT P.IDAPLIC
            FROM DESCARREGA D, PRODUCTE P
            WHERE D.IDPRODUCTE = P.CODIPRODUCTE 
                  AND  Extract(YEAR FROM D.DATADESC) = e_anyo
                  AND d.idpaisregusu = e_pais);
  
-- ACTUALITZAR TOTES LES DADES
      
      -- MIRAR SI EXISTEIX LA FILA DE LA TAULA ESTADISTICA_678
                SELECT COUNT(1) INTO v_Fila 
                FROM ESTADISTICA_678
                WHERE ANYO = e_anyo AND IDPAIS = e_pais;
                
                -- SI EXISTEIX ACTUALITZEM LES DADES (UPDATE)
                    IF( v_Fila >0) THEN
                        UPDATE estadistica_678
                          SET numusumindes = v_numusumindes,
                              INGRESSOSANYPAIS = v_INGRESSOSANYPAIS,
                              NUMAPLICMINDES = v_NUMAPLICMINDES
                              WHERE ANYO = e_anyo AND IDPAIS = e_pais;
                     ELSE
                -- SI NO EXISTEIX CREEM LA FILA (INSERT)
                        INSERT INTO ESTADISTICA_678
                          VALUES (e_anyo,e_pais,v_numusumindes,v_INGRESSOSANYPAIS,v_NUMAPLICMINDES);
                     END IF;
     
-- FINALITZA AMB EXIT
       	e_rsp := 'OK'; 
        DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||e_rsp);
             
 -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
        COMMIT;
 END IF;
COMMIT;
 EXCEPTION      
 
       WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: ELS PARAMETRES ENTRADA NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
        
      WHEN EX_ANY_NOK THEN   -- CONTROL QUE EXISTEIXIN DESCARREGUES D'AQUEST ANY: e_anyo
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: NO EXISTEIXIN DESCARREGUES EN ANY: '||e_anyo||' '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);

      WHEN EX_PAIS_NUL THEN  -- CONTROL PAIS EXISTEIXI
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: AQUEST PAIS NO EXISTEIX '||v_error|| ': '||v_desc;

           -- INSERTEM L'ERROR A LA TAULA DE LOG
              ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
              DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);               
                      
      WHEN EX_PAIS_NOK THEN   -- CONTROL QUE EXISTEIXIN DESCARREGUES D'AQUEST PAIS: e_pais
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: NO EXISTEIXIN DESCARREGUES EN PAIS: '||e_pais||' '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
               
      WHEN NO_DATA_FOUND THEN  -- CONTROL DADES CORRECTES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR: DADA NO EXISTEIX '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);
      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               e_rsp := v_prodediment||' ERROR ! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada,'RSP: '||e_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||e_rsp);   
END MODUL_EST_678;

/
spool OFF