--------------------------------------------------------
--  DDL for Procedure CONSULTA_1
--------------------------------------------------------

spool CARREGAR_CONSULTES.log

set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."CONSULTA_1" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: CONSULTA_1.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         CONSULTA_1
   DESCRIPCI:          LLISTA LES DADES CORRESPONENTS A:
                        El llistat de tots els desenvolupadors dun pas donat amb
                        totes les seves dades, incloent el nmero daplicacions
                        diferents publicades, entenent com a aplicacions: 
                        APLICACIO + SSOO
   PARAMETRES ENTRADA:  c_pais
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA NO POT SER NULL 
                        ERROR: EL PAIS NO EXISTEIX
   EXEMPLE DE CRIDA:    CONSULTA_1(1,rsp);
********************************************************************************************/
                        c_pais  IN DESENVOLUPADOR.IDPAISREGISTRE%TYPE,
                        c_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 
/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_pais              DESENVOLUPADOR.IDPAISREGISTRE%TYPE;
    v_ntest             number;

    v_iddesenv          DESENVOLUPADOR.IDDESENV%TYPE;
    v_nomdesen          DESENVOLUPADOR.NOMDESEN%TYPE;
    v_codiident         DESENVOLUPADOR.CODIIDENT%TYPE; 
    v_adreca            DESENVOLUPADOR.ADRECA%TYPE;
    v_localitat         LOCALITAT.NOMLOCALITAT%TYPE; 
    v_provincia         PROVINCIA.NOMPROVINCIA%TYPE;
    v_codipostal        DESENVOLUPADOR.CODIPOSTAL%TYPE;
    v_numtelefon        DESENVOLUPADOR.NUMTEL%TYPE;
    v_paginaweb         DESENVOLUPADOR.PAGINAWEB%TYPE;
    v_email             DESENVOLUPADOR.EMAIL%TYPE;
    v_paisregistre      PAIS.CODIISO%TYPE;
    v_representant      VARCHAR2(40);
    v_tt_des            number;
    v_nompais           PAIS.NOMPAIS%TYPE;
    
    CURSOR C1 IS 
                  SELECT  D.IDDESENV,
                          D.NOMDESEN,
                          D.CODIIDENT,
                          D.ADRECA,
                          L.NOMLOCALITAT,
                          D.CODIPOSTAL,
                          PROV.NOMPROVINCIA,
                          PAIS.CODIISO,
                          SUBSTR(R.NOM||' '||R.COGNOM1||' '||R.COGNOM2,1,40),
                          D.NUMTEL,
                          D.PAGINAWEB,
                          D.EMAIL,
                          Q2.TT_DES
                  FROM DESENVOLUPADOR D, LOCALITAT L, PROVINCIA PROV,
                       PAIS, REPRESENTANT R,
                      (SELECT Q1.IDDESEV   AS ID_DES,
                        COUNT(Q1.T_DESENV) AS TT_DES
                          FROM
                            (SELECT AUTOR.IDDESEV,
                              PRODUCTE.IDAPLIC,
                              PRODUCTE.IDSSOO,
                              COUNT(1) AS T_DESENV
                            FROM PRODUCTE,
                              AUTOR
                            WHERE AUTOR.IDPROD = PRODUCTE.CODIPRODUCTE
                            GROUP BY AUTOR.IDDESEV,
                              PRODUCTE.IDAPLIC,
                              PRODUCTE.IDSSOO
                            ) Q1
                            GROUP BY Q1.IDDESEV
                      ) Q2
                  WHERE L.IDLOCALITAT  = D.IDLOCALITAT
                  AND PROV.IDPROVINCIA = L.IDPROVINCIA
                  AND PAIS.IDPAIS      = D.IDPAISREGISTRE
                  AND PAIS.IDPAIS      = PROV.IDPAIS
                  AND D.IDREPRESENTANT = R.IDREPRESENTANT
                  AND (D.ESTAT         = 'A')
                  AND D.IDDESENV       = Q2.ID_DES
                  AND D.IDPAISREGISTRE = c_pais;
  
    
    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL           EXCEPTION;  -- no existeix la clau forana
    EX_PR_NOK           EXCEPTION;  -- el pais de registre del desenvolupador incorrecte 
    EX_DES_NOK          EXCEPTION;  -- no hi han desenvolupadors d'aquest pais
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

BEGIN

     v_prodediment :=   'CONSULTA 1 ';	
     v_param_entrada := 'PAIS: '||c_pais;

-- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (c_pais IS NULL ) THEN
            RAISE EX_INS_NUL;
        END IF; 

  -- MIRAR SI EL PAIS ES CORRECTE I 
        SELECT NOMPAIS, COUNT(1) INTO v_nompais, v_ntest
           FROM PAIS
            WHERE IDPAIS = c_pais
            GROUP BY NOMPAIS;
        IF (v_ntest=0) THEN
           RAISE EX_PR_NOK;
        END IF;          
        
  -- MIRAR SI HI HAN DESESENVOLUPADORS ASSOCIATS A AQUEST PAIS
        SELECT COUNT(1) INTO v_ntest
           FROM DESENVOLUPADOR
            WHERE IDPAISREGISTRE = c_pais;
        IF (v_ntest=0) THEN
           RAISE EX_DES_NOK;
        END IF;

        DBMS_OUTPUT.PUT_LINE (' ');
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('El llistat de tots els desenvolupadors incloent el nmero daplicacions diferents publicades,   ');
        DBMS_OUTPUT.PUT_LINE ('entenent com a aplicacions:  APLICACIO + SSOO');
        DBMS_OUTPUT.PUT_LINE ('Pais de registre dels desenvolupadors :'||v_nompais);
        DBMS_OUTPUT.PUT_LINE ('Data del llistat :'||to_char(sysdate,'Day, DD "de" MONTH "de" YYYY'));
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');

        DBMS_OUTPUT.PUT_LINE (rpad('Codi',5)
                            ||' '||rpad('Nom Desenvolupador',25)
                            ||' '||rpad('Identif.',9)
                            ||' '||rpad('Adrea',30)
                            ||' '||rpad('Localitat',15)
                            ||' '||rpad('C.P.',5)
                            ||' '||rpad('Provincia',15)
                            ||' '||rpad('Pais',4)
                            ||' '||rpad('Representant',20)
                            ||' '||rpad('Telefon',9)
                            ||' '||rpad('Pag. Web',20)
                            ||' '||rpad('E-mail',20)
                            ||' '||rpad('NAp.publ.',10));
         DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');                   
        
    -- APERTURA DEL CURSOR
          OPEN C1;
    -- EXTRACCIO DE LES DADES
          LOOP
          FETCH C1 INTO
                         v_iddesenv, v_nomdesen , v_codiident, v_adreca,  v_localitat, v_codipostal, 
                         v_provincia, v_paisregistre, v_representant, v_numtelefon , v_paginaweb,
                         v_email, v_tt_des;
          EXIT WHEN C1%NOTFOUND;
          DBMS_OUTPUT.PUT_LINE (rpad(v_iddesenv,5)
                            ||' '||rpad(v_nomdesen,25)
                            ||' '||rpad(v_codiident,9)
                            ||' '||rpad(v_adreca,30)
                            ||' '||rpad(v_localitat,15)
                            ||' '||rpad(v_codipostal,5)
                            ||' '||rpad(v_provincia,15)
                            ||' '||rpad(v_paisregistre,4)
                            ||' '||rpad(v_representant,20)
                            ||' '||rpad(v_numtelefon,9)
                            ||' '||rpad(v_paginaweb,20)
                            ||' '||rpad(v_email,20)
                            ||' '||rpad(v_tt_des,10));
          DBMS_OUTPUT.PUT_LINE (' ');
          END LOOP;
          DBMS_OUTPUT.PUT_LINE (' ');
        
    -- TANCAR EL CURSOR   
          CLOSE C1;

    -- FINALITZA AMB EXIT
          	c_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||c_rsp);
            
    -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);        
      
      COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
             
          WHEN EX_PR_NOK THEN   -- CONTROL PAIS REGISTRE 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: PAIS DE REGISTRE INCORRECTE '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);     
         
          WHEN EX_DES_NOK THEN   -- CONTROL DESENVOPADORS D'AQUEST PAIS REGISTRE 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: NO HI HAN DESESENVOLUPADORS ASSOCIATS A AQUEST PAIS '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);          

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR !!! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

END CONSULTA_1;

/
--------------------------------------------------------
--  DDL for Procedure CONSULTA_2
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."CONSULTA_2" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: CONSULTA_2.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         CONSULTA_2
   DESCRIPCI:          LLISTA LES DADES CORRESPONENTS A:
                        El llistat de totes les aplicacions actives i de les seves dades
                        principals, ordenat pel nmero total de descrregues que
                        han tingut fins al moment a nivell mundial, 
                        entenent com aplicacions: APLICACIO + SSOO + VERSIO
   PARAMETRES ENTRADA:  SENSE PARAMETRES
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: HI VALORS QUE NO PODEN SER NULL
                        ERROR: NO HI HA APLICACIONS ACTIVES
   EXEMPLE DE CRIDA:    CONSULTA_2(rsp);
********************************************************************************************/
                        c_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 
/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_ntest             number;

    v_idproducte        PRODUCTE.CODIPRODUCTE%TYPE;
    v_nomapli           APLICACIO.NOMAPLI%TYPE;
    v_ssoo              SSOO.NOMSSOO%TYPE; 
    v_versio            PRODUCTE.VERSIO%TYPE;
    v_midaaplic         PRODUCTE.MIDAAPLIC%TYPE;
    v_resolucio         PRODUCTE.RESOLUCIO%TYPE;
    v_datapujada        PRODUCTE.DATAPUJADA%TYPE;
    v_tt_des            number;
    
    CURSOR C2 IS 
                  SELECT  D.IDPRODUCTE,
                          APLICACIO.NOMAPLI,
                          SSOO.NOMSSOO,
                          P.VERSIO,
                          P.MIDAAPLIC,
                          P.RESOLUCIO,
                          P.DATAPUJADA,
                          COUNT(1) AS T_DESCAR
                  FROM  DESCARREGA D,
                        PRODUCTE P,
                        APLICACIO,
                        SSOO
                  WHERE D.IDPRODUCTE        = P.CODIPRODUCTE
                        AND APLICACIO.IDAPLICACIO = P.IDAPLIC
                        AND SSOO.IDSSOO           = P.IDSSOO
                        AND (P.ESTATVERSIO        = 'A')
                  GROUP BY  D.IDPRODUCTE,
                            APLICACIO.NOMAPLI,
                            SSOO.NOMSSOO,
                            P.VERSIO,
                            P.MIDAAPLIC,
                            P.RESOLUCIO,
                            P.DATAPUJADA
                  ORDER BY COUNT(1) DESC;
  
    
    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL           EXCEPTION;  -- no existeix la clau forana
    EX_PR_NOK           EXCEPTION;  -- el pais de registre del desenvolupador incorrecte 
    EX_PRO_NOK          EXCEPTION;  -- no hi han aplicacions actives
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);

BEGIN

     v_prodediment :=   'CONSULTA 2 ';	
     v_param_entrada := 'SENSE PARAMETRES ENTRADA';

  -- MIRAR SI HI HAN HAN APLICACIONS ACTIVES
        SELECT COUNT(1) INTO v_ntest
           FROM PRODUCTE
            WHERE ESTATVERSIO = 'A';
        IF (v_ntest=0) THEN
           RAISE EX_PRO_NOK;
        END IF;

        DBMS_OUTPUT.PUT_LINE (' ');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('El llistat de totes les aplicacions actives i de les seves dades principals, ordenat pel nmero ');
        DBMS_OUTPUT.PUT_LINE ('total de descrregues que han tingut fins al moment a nivell mundial, ');
        DBMS_OUTPUT.PUT_LINE ('entenent com aplicacions:  APLICACIO + SSOO + VERSIO');
        DBMS_OUTPUT.PUT_LINE ('Data del llistat :'||to_char(sysdate,'Day, DD "de" MONTH "de" YYYY'));
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------------------------');

        DBMS_OUTPUT.PUT_LINE (rpad('Codi',5)
                            ||' '||rpad('Nom Aplicaci',25)
                            ||' '||rpad('Sistema Operatiu',12)
                            ||' '||rpad('Versio',10)
                            ||' '||rpad('Mida',15)
                            ||' '||rpad('Resolucio',15)
                            ||' '||rpad('Data Pujada',15)
                            ||' '||rpad('NAp.desc.',10));
         DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------------------------');                   
        
    -- APERTURA DEL CURSOR
          OPEN C2;
    -- EXTRACCIO DE LES DADES
          LOOP
          FETCH C2 INTO
                         v_idproducte, v_nomapli , v_ssoo, v_versio,  v_midaaplic, v_resolucio, 
                         v_datapujada, v_tt_des;
          EXIT WHEN C2%NOTFOUND;
          DBMS_OUTPUT.PUT_LINE (rpad(v_idproducte,5)
                            ||' '||rpad(v_nomapli,25)
                            ||' '||rpad(v_ssoo,12)
                            ||' '||rpad(v_versio,10)
                            ||' '||rpad(v_midaaplic,15)
                            ||' '||rpad(v_resolucio,15)
                            ||' '||rpad(v_datapujada,15)
                            ||' '||rpad(v_tt_des,10));
                            
          DBMS_OUTPUT.PUT_LINE (' ');
          END LOOP;
          DBMS_OUTPUT.PUT_LINE (' ');
        
     -- TANCAR EL CURSOR   
          CLOSE C2;

     -- FINALITZA AMB EXIT
          	c_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||c_rsp);
            
     -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);        
      
      COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: HI VALORS QUE NO PODEN SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
               
          WHEN EX_PR_NOK THEN   -- CONTROL PAIS REGISTRE 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: PAIS DE REGISTRE INCORRECTE '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);     
         
          WHEN EX_PRO_NOK THEN   -- CONTROL aplicacions actives
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: no hi han aplicacions actives '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);          

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR !!! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
END CONSULTA_2;

/
--------------------------------------------------------
--  DDL for Procedure CONSULTA_3
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."CONSULTA_3" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: CONSULTA_3.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         CONSULTA_3
   DESCRIPCI:          LLISTA LES DADES CORRESPONENTS A:
                        Donada una aplicaci i un any concret: el llistat de tots els
                        pasos on sha descarregat aquell any, aix com el nmero
                        de descrregues que ha tingut a cada pas, 
                        entenent com aplicacions:  APLICACIO (sense tenir en compte
                        ni el Sistema Operatiu ni la versi)
   PARAMETRES ENTRADA:  c_idaplicacio, c_any
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: ELS PARAMETRES ENTRADA NO PODEN SER NULL 
                        ERROR: L'APLICACIO NO EXISTEIX
                        ERROR: NO HI HA DESCARREGUES ASSOCIADES A AQUEST ANY
   EXEMPLE DE CRIDA:    CONSULTA_3(3,2012,rsp);
********************************************************************************************/
                        c_idaplicacio  IN APLICACIO.IDAPLICACIO%TYPE,
                        c_any          IN NUMBER,
                        c_rsp          OUT NOCOPY logs.paramout%TYPE   )
AS 
/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_pais              PAIS.IDPAIS%TYPE;
    v_ntest             number;

    v_nomapli           APLICACIO.NOMAPLI%TYPE;
    v_nompais           PAIS.NOMPAIS%TYPE;
    v_tt_des            number;
    
    CURSOR C3 IS 
                      SELECT  AP.NOMAPLI,
                              PA.NOMPAIS,
                              COUNT(1) AS T_DESCAR
                      FROM DESCARREGA D,
                            PAIS PA,
                            APLICACIO AP,
                            PRODUCTE P
                      WHERE PA.IDPAIS                          = D.IDPAISREGUSU
                            AND AP.IDAPLICACIO                = P.IDAPLIC
                            AND D.IDPRODUCTE                  = P.CODIPRODUCTE
                            AND (AP.IDAPLICACIO               = c_idaplicacio
                            AND Extract(YEAR FROM D.DATADESC) = c_any)
                      GROUP BY AP.NOMAPLI, PA.NOMPAIS;
  
    
    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL           EXCEPTION;  -- no existeix la clau forana
    EX_ANYO_NOK         EXCEPTION;  -- no hi ha cap descarrega en aquest any
    EX_APLI_NOK         EXCEPTION;  -- no hi ha cap aplicacio amb aquest codi
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);


BEGIN

     v_prodediment :=   'CONSULTA 3 ';	
     v_param_entrada := 'CODI APLICACIO: '||c_idaplicacio||
                        ' / ANY : '||c_any;

  -- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (c_idaplicacio IS NULL OR c_any IS NULL) THEN
            RAISE EX_INS_NUL;
        END IF; 

   -- MIRAR SI L'APLICACIO EXISTEIX I OBTE EL NOM 
       SELECT  NOMAPLI , COUNT(1)INTO v_nomapli, v_ntest
          FROM APLICACIO
          WHERE IDAPLICACIO = c_idaplicacio
          GROUP BY NOMAPLI;
        IF (v_ntest=0) THEN
           RAISE EX_APLI_NOK;
        END IF;          
    -- MIRAR SI HI HAN DESCARREGUES ASSOCIADES A AQUEST ANY
        SELECT COUNT(1) INTO v_ntest
           FROM DESCARREGA
            WHERE Extract(YEAR FROM DESCARREGA.DATADESC) = c_any;
        IF (v_ntest=0) THEN
           RAISE EX_ANYO_NOK;
        END IF;

        DBMS_OUTPUT.PUT_LINE (' ');
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Donada una aplicaci i un any concret: el llistat de tots els pasos on sha descarregat aquell ');
        DBMS_OUTPUT.PUT_LINE ('any, aix com el nmero de descrregues que ha tingut a cada pas, ');
        DBMS_OUTPUT.PUT_LINE ('entenent com aplicacions:  APLICACIO (sense tenir en compte');
        DBMS_OUTPUT.PUT_LINE (' ni el Sistema Operatiu ni la versi)   ');
        DBMS_OUTPUT.PUT_LINE ('CODI APLICACIO :'||c_idaplicacio||' NOM APLICACIO: '||v_nomapli||'  ANY: '||c_any);
        DBMS_OUTPUT.PUT_LINE ('Data del llistat :'||to_char(sysdate,'Day, DD "de" MONTH "de" YYYY'));
        DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');

        DBMS_OUTPUT.PUT_LINE (rpad('Pais',20)
                       ||' '||rpad('Nm. descarregues',20));
         DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------------');                   
        
      -- APERTURA DEL CURSOR
          OPEN C3;
      -- EXTRACCIO DE LES DADES
          LOOP
          FETCH C3 INTO
                         v_nomapli, v_nompais , v_tt_des;

          EXIT WHEN C3%NOTFOUND;
          DBMS_OUTPUT.PUT_LINE (rpad(v_nompais,20)
                         ||' '||rpad(v_tt_des,20));
                            
          DBMS_OUTPUT.PUT_LINE (' ');
          END LOOP;
          DBMS_OUTPUT.PUT_LINE (' ');
        
       -- TANCAR EL CURSOR   
          CLOSE C3;

      -- FINALITZA AMB EXIT
          	c_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||c_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);        
      
      COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: ELS PARAMETRES ENTRADA NO PODEN SER NULL'||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

          WHEN DUP_VAL_ON_INDEX THEN  -- CONTROL CLAUS UNIQUES NO PODEN ESTAR DUPLICADES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: CLAU UNICA, NO POT HAVER-HI DUPLICATS '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
               
          WHEN EX_ANYO_NOK THEN   -- CONTROL ANY ENTRADA
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: no hi ha cap descarrega en aquest any '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);     
         
          WHEN EX_APLI_NOK THEN   -- CONTROL MIRAR SI L'APLICACIO ES CORRECTA
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: no hi ha cap aplicacio amb aquest codi '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);          

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR !!! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

END CONSULTA_3;

/
--------------------------------------------------------
--  DDL for Procedure CONSULTA_4
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."CONSULTA_4" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: CONSULTA_4.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         CONSULTA_4
   DESCRIPCI:          LLISTA LES DADES CORRESPONENTS A:
                        Donat un usuari final (identificat pel seu nmero de telfon),
                        el llistat de tota la seva activitat de descrregues a la
                        plataforma, incloent data, aplicaci descarregada, preu que
                        va pagar, etc..., entenent com a aplicacions: 
                        APLICACIO + SSOO + VERSIO
   PARAMETRES ENTRADA:  c_ntel
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA NO POT SER NULL 
                        ERROR: EL TELEFON NO EXISTEIX    
                        ERROR: EL NUMERO DE DIGITS DEL TELEFON ES INCORRECTE  
                        ERROR: l'USUARI D'AQUEST TELEFON NO TE DESCARREGUES
   EXEMPLE DE CRIDA:    CONSULTA_4('649665511',rsp);
********************************************************************************************/
                        c_ntel  IN USUARIS.NUMTELEFON%TYPE,
                        c_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 
/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_pais              DESENVOLUPADOR.IDPAISREGISTRE%TYPE;
    v_ntest             number;
    v_idUs              USUARIS.IDUSUARI%TYPE;
    v_nomUs             VARCHAR(70);

    v_datadesc          DESCARREGA.DATADESC%TYPE;
    v_idproducte        DESCARREGA.IDPRODUCTE%TYPE;
    v_nomaplic          APLICACIO.NOMAPLI%TYPE; 
    v_nomssoo           SSOO.NOMSSOO%TYPE;
    v_versio            PRODUCTE.VERSIO%TYPE;
    v_preupagat         DESCARREGA.PREUPAGAT%TYPE; 
    v_nomoper           OPERADORS.NOMOPERADOR%TYPE;
    v_nommodel          MODEL_DISPOSITIU.NOMMODEL%TYPE;
    v_formapago         FORMA_PAGO.NOMFORMAPAGO%TYPE;
    
    CURSOR C4 IS 
                      SELECT    U.IDUSUARI,
                                D.DATADESC,
                                D.IDPRODUCTE,
                                AP.NOMAPLI,
                                S.NOMSSOO,
                                P.VERSIO,
                                D.PREUPAGAT,
                                O.NOMOPERADOR,
                                M.NOMMODEL,
                                F.NOMFORMAPAGO
                      FROM  DESCARREGA D,
                            USUARIS U,
                            DISPOSITIU DI,
                            MODEL_DISPOSITIU M,
                            OPERADORS O,
                            FORMA_PAGO F,
                            SSOO S,
                            PRODUCTE P,
                            APLICACIO AP
                      WHERE D.IDUSUARI    = U.IDUSUARI
                              AND P.CODIPRODUCTE  = D.IDPRODUCTE
                              AND DI.IDDISPOSITIU = D.IDDISPOSITIU
                              AND M.IDMODEL       = DI.IDMODEL
                              AND O.IDOPERADOR    = D.IDOPDES
                              AND F.IDFORMAPAGO   = D.IDPAGAMENT
                              AND AP.IDAPLICACIO  = P.IDAPLIC
                              AND S.IDSSOO        = P.IDSSOO
                              AND U.NUMTELEFON    = c_ntel
                      ORDER BY D.DATADESC;

    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_NTEL_NUL         EXCEPTION;  -- el telefon no correspon a cap usuari 
    EX_NTEL_NOK         EXCEPTION;  -- el nmero de digits del telefon es incorrecte
    EX_DES_NOK          EXCEPTION;  -- no hi han descarregues d'aquest usuari
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN

     v_prodediment :=   'CONSULTA 4 ';	
     v_param_entrada := 'NUM. TELEFON: '||c_ntel;

 -- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (c_ntel IS NULL ) THEN
            RAISE EX_INS_NUL;
        END IF; 

 -- VERIFIQUEM EL NUMERO DE DIGITS DEL TELEFON SIGUI CORRECTE  
        IF (LENGTH(c_ntel) <> 9 ) THEN
            RAISE EX_NTEL_NOK;
        END IF; 
 
 -- MIRAR SI EL TELEFON ES CORRECTE 
        SELECT COUNT(1) INTO v_ntest
           FROM USUARIS
            WHERE NUMTELEFON = c_ntel;
            
         IF (v_ntest=0) THEN
           RAISE EX_NTEL_NUL;
         END IF;          
        
 -- OBTENIM EL CODI DE L'USUARI     
        SELECT IDUSUARI, SUBSTR(NOMUSUARI||' '||COGNOM1||' '||COGNOM2,1,70) INTO v_idUs, v_nomUs
           FROM USUARIS
            WHERE NUMTELEFON = c_ntel;
                    
 -- MIRAR SI HI HAN DESCARREGUES ASSOCIADES A AQUEST USUARI
        SELECT  COUNT(1) INTO  v_ntest
           FROM DESCARREGA
            WHERE IDUSUARI = v_idUs;
        IF (v_ntest=0) THEN
           RAISE EX_DES_NOK;
        END IF;

        DBMS_OUTPUT.PUT_LINE (' ');
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Donat un usuari final (identificat pel seu nmero de telfon), el llistat de tota la seva ');
        DBMS_OUTPUT.PUT_LINE ('activitat de descrregues a la plataforma, incloent data, aplicaci descarregada, preu que ');
        DBMS_OUTPUT.PUT_LINE ('va pagar, el dipositiu utilitzat, operador telefonic utilitzat i la forma de pagament, ');
        DBMS_OUTPUT.PUT_LINE ('entenent com a aplicacions:  APLICACIO + SSOO + VERSIO');
        DBMS_OUTPUT.PUT_LINE ('Nmero de telfon :'||c_ntel||' Nom Usuari: '||v_nomUs);
        DBMS_OUTPUT.PUT_LINE ('Data del llistat :'||to_char(sysdate,'Day, DD "de" MONTH "de" YYYY'));
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------');

        DBMS_OUTPUT.PUT_LINE (rpad(' Data',8)
                            ||' '||rpad(' Codi ',6)
                            ||' '||rpad(' Aplicacio ',20)
                            ||' '||rpad('SS OO',10)
                            ||' '||rpad('Versio',8)
                            ||' '||rpad('Preu pagat',12)
                            ||' '||rpad('Operador',10)
                            ||' '||rpad('Model dispositiu',10)
                            ||' '||rpad('F.Pagament',10));
         DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------');                   
        
      -- APERTURA DEL CURSOR
          OPEN C4;
      -- EXTRACCIO DE LES DADES
          LOOP
          FETCH C4 INTO
                         v_idUs, v_datadesc , v_idproducte, v_nomaplic,  v_nomssoo, v_versio, 
                         v_preupagat, v_nomoper, v_nommodel, v_formapago;
          EXIT WHEN C4%NOTFOUND;
          DBMS_OUTPUT.PUT_LINE (rpad(v_datadesc,8)
                            ||' '||rpad(v_idproducte,6)
                            ||' '||rpad(v_nomaplic,20)
                            ||' '||rpad(v_nomssoo,10)
                            ||' '||rpad(v_versio,8)
                            ||' '||rpad(TO_CHAR(v_preupagat,'999,990.99'),12)
                            ||' '||rpad(v_nomoper,10)
                            ||' '||rpad(v_nommodel,10)
                            ||' '||rpad(v_formapago,10));
          DBMS_OUTPUT.PUT_LINE (' ');
          END LOOP;
          DBMS_OUTPUT.PUT_LINE (' ');
        
       -- TANCAR EL CURSOR   
          CLOSE C4;

      -- FINALITZA AMB EXIT
          	c_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||c_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);        
      
      COMMIT;
        
  EXCEPTION 
       
        WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
          
       WHEN EX_NTEL_NUL THEN   -- CONTROL NUMERO TELEFON
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: El telefon no correspon a cap usuari  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);  

       WHEN EX_NTEL_NOK THEN   -- CONTROL NUMERO DE DIGITS DEL TELEFON
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: El nmero de digits del telefon es incorrecte '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);     
         
      WHEN EX_DES_NOK THEN   -- CONTROL aquest usuari no te descarregues 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: aquest usuari no te descarregues  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);          

      WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR !!! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

END CONSULTA_4;

/
--------------------------------------------------------
--  DDL for Procedure CONSULTA_5
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."CONSULTA_5" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: CONSULTA_5.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         CONSULTA_5
   DESCRIPCI:          LLISTA LES DADES CORRESPONENTS A:
                        Donat un any concret el llistat dels 20 usuaris que ms
                        diners shan gastat en aplicacions mbils, ordenat de ms a menys, 
                        entenent com aplicacions:  APLICACIO + SSOO + VERSIO)
   PARAMETRES ENTRADA:  c_any
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA NO POT SER NULL 
                        ERROR: NO HI HA CAP DESCARREGA EN AQUEST ANY                       
   EXEMPLE DE CRIDA:    CONSULTA_5(2012,rsp);
********************************************************************************************/
                        c_any          IN NUMBER,
                        c_rsp          OUT NOCOPY logs.paramout%TYPE   )
AS 
/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_pais              PAIS.IDPAIS%TYPE;
    v_ntest             number;
    
    v_idUs              USUARIS.IDUSUARI%TYPE;
    v_nomUsu            VARCHAR2(40);
    v_nif               USUARIS.NIF%TYPE; 
    v_numtelefon        USUARIS.NUMTELEFON%TYPE;
    v_adreca            USUARIS.ADRECA%TYPE;
    v_localitat         LOCALITAT.NOMLOCALITAT%TYPE; 
    v_codipostal        USUARIS.CODIPOSTAL%TYPE;    
    v_provincia         PROVINCIA.NOMPROVINCIA%TYPE;
    v_nompais           PAIS.NOMPAIS%TYPE;
    v_tt_des            number;
    v_NF                number;
    CURSOR C5 IS 
                        SELECT D.IDUSUARI,
                          SUBSTR(U.NOMUSUARI
                          || ' '
                          || U.COGNOM1
                          || ' '
                          || U.COGNOM2,1,40) nomUsu,
                          U.NIF,
                          U.NUMTELEFON,
                          U.ADRECA,
                          L.NOMLOCALITAT,
                          U.CODIPOSTAL,
                          P.NOMPROVINCIA,
                          PA.NOMPAIS,
                          SUM(D.PREUPAGAT) AS total_PREUS
                        FROM DESCARREGA D,
                          USUARIS U,
                          LOCALITAT L,
                          PROVINCIA P,
                          PAIS PA
                        WHERE D.IDUSUARI                   = U.IDUSUARI
                        AND U.IDLOCALITAT                  = L.IDLOCALITAT
                        AND L.IDPROVINCIA                  = P.IDPROVINCIA
                        AND P.IDPAIS                       = PA.IDPAIS
                        AND (Extract(YEAR FROM D.DATADESC) = c_any          )
                        GROUP BY D.IDUSUARI,
                          U.NIF,
                          U.NUMTELEFON,
                          U.ADRECA,
                          L.NOMLOCALITAT,
                          U.CODIPOSTAL,
                          P.NOMPROVINCIA,
                          PA.NOMPAIS,
                          U.NOMUSUARI,
                          U.COGNOM1,
                          U.COGNOM2
                        ORDER BY SUM(D.PREUPAGAT) DESC;
    
    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_ANYO_NOK         EXCEPTION;  -- no hi ha cap descarrega en aquest any
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);

BEGIN

     v_prodediment :=   'CONSULTA 5 ';	
     v_param_entrada := 'ANY : '||c_any;
     v_NF:= 20;   --  NUMERO DE FILES QUE RELACIONARA EL LLISTAT

-- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (c_any IS NULL ) THEN
            RAISE EX_INS_NUL;
        END IF; 

-- MIRAR SI HI HAN DESCARREGUES ASSOCIADES A AQUEST ANY
        SELECT COUNT(1) INTO v_ntest
           FROM DESCARREGA
            WHERE Extract(YEAR FROM DESCARREGA.DATADESC) = c_any;
        IF (v_ntest=0) THEN
           RAISE EX_ANYO_NOK;
        END IF;

        DBMS_OUTPUT.PUT_LINE (' ');
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Donat un any concret el llistat dels 20 usuaris que ms diners shan gastat en aplicacions mbils,');
        DBMS_OUTPUT.PUT_LINE ('ordenat de ms a menys, entenent com aplicacions:  APLICACIO + SSOO + VERSIO ');
        DBMS_OUTPUT.PUT_LINE ('ANY: '||c_any);
        DBMS_OUTPUT.PUT_LINE ('Data del llistat :'||to_char(sysdate,'Day, DD "de" MONTH "de" YYYY'));
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------------------------------------------');

        DBMS_OUTPUT.PUT_LINE (rpad('Codi',5)
                       ||' '||rpad('Nom Usuari',30)
                       ||' '||rpad('NIF ',12)
                       ||' '||rpad('Telefon',9)                       
                       ||' '||rpad('Adrea',20)
                       ||' '||rpad('Localitat',15)
                       ||' '||rpad('C.P.',5)
                       ||' '||rpad('Provincia',15)
                       ||' '||rpad('Pais',14)
                       ||' '||rpad('Import pagat',12)
                       ||' '||rpad('Moneda',6));
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------------------------------------------');                   
        
      -- APERTURA DEL CURSOR
          OPEN C5;
      -- EXTRACCIO DE LES DADES
          LOOP
          FETCH C5 INTO
                         v_idUs, v_nomUsu , v_nif, v_numtelefon,v_adreca, 
                         v_localitat, v_codipostal, v_provincia, v_nompais, v_tt_des;

          EXIT WHEN C5%NOTFOUND or C5%ROWCOUNT= v_NF+1;
          DBMS_OUTPUT.PUT_LINE (rpad(v_idUs,5)
                       ||' '||rpad(v_nomUsu,30)
                       ||' '||rpad(v_nif,12)
                       ||' '||rpad(v_numtelefon,9)                       
                       ||' '||rpad(v_adreca,20)
                       ||' '||rpad(v_localitat,15)
                       ||' '||rpad(v_codipostal,5)
                       ||' '||rpad(v_provincia,15)
                       ||' '||rpad(v_nompais,14)          
                       ||' '||rpad(TO_CHAR(v_tt_des,'999,990.99'),12)
                       ||' '||rpad('EUR',3));
                            
          DBMS_OUTPUT.PUT_LINE (' ');
          END LOOP;
          DBMS_OUTPUT.PUT_LINE (' ');
        
       -- TANCAR EL CURSOR   
          CLOSE C5;

      -- FINALITZA AMB EXIT
          	c_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||c_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);        
      
      COMMIT;
        
  EXCEPTION 
      WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

               
     WHEN EX_ANYO_NOK THEN   -- CONTROL ANY ENTRADA
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: no hi ha cap descarrega en aquest any '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);     
         
     WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR !!! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
END CONSULTA_5;

/
--------------------------------------------------------
--  DDL for Procedure CONSULTA_6
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APPS"."CONSULTA_6" (

/************************************************************************************
   Alumna: Merc Jorquera Teixid
   Base de dades: Oracle 11g
   TFC: DESCARREGA D'APLICACIONS PER A MBILS INTEL.LIGENTS
   SCRIPT: CONSULTA_6.sql
************************************************************************************/

/********************************************************************************************
   PROCEDIMENT:         CONSULTA_6
   DESCRIPCI:          LLISTA LES DADES CORRESPONENTS A:
                        Donada una determinada categoria, el llistat de les aplicacions actives
                        per descarregar relacionadees amb aquesta categoria, entenent 
                        com aplicacions: APLICACIO + SSOO + VERSIO
   PARAMETRES ENTRADA:  c_idcategoria
   PARAMETRES SORTIDA:  rsp
   RETORNA:             'OK' SI L'EXECUCI HA ACABAT CORRECTAMENT 
                        'ERROR: tipus d'error' SI HA FRACASAT
   ERRORS:              ERROR: EL PARAMETRE ENTRADA NO POT SER NULL 
                        ERROR: LA CATEGORIA c_idcategoria  NO EXISTEIX
                        ERROR: NO HI HA CAP PRODUCTE ACTIU D'AQUESTA CATEGORIA                             
   EXEMPLE DE CRIDA:    CONSULTA_6('1',rsp);
********************************************************************************************/
                        c_idcategoria  IN CATEGORIA.IDCATEGORIA%TYPE,
                        c_rsp   OUT NOCOPY logs.paramout%TYPE   )
AS 
/* VARIABLES LOCALS */
    v_prodediment       LOGS.PROCEDIMENT%TYPE;
    v_param_entrada     LOGS.PARAMIN%TYPE; 
    v_error             number;
    v_desc              VARCHAR2(256);
    v_ntest             number;
    v_idcateg           CATEGORIA.IDCATEGORIA%TYPE;
    v_nomcateg          CATEGORIA.NOMCATEGORIA%TYPE;

    v_datapujada        PRODUCTE.DATAPUJADA%TYPE;
    v_idproducte        PRODUCTE.CODIPRODUCTE%TYPE;
    v_nomaplic          APLICACIO.NOMAPLI%TYPE; 
    v_nomssoo           SSOO.NOMSSOO%TYPE;
    v_versio            PRODUCTE.VERSIO%TYPE;
    v_midaapli          PRODUCTE.MIDAAPLIC%TYPE;
    v_resoulucio        PRODUCTE.RESOLUCIO%TYPE;
    v_idcat             CATEGORIA.IDCATEGORIA%TYPE;
    
    --- DECLARACIO DEL CURSOR, EN AQUEST CAS HE UTILITZAT UNA VISTA (VIEW)
    CURSOR C6 IS 
                      SELECT * FROM V6
                      WHERE IDCATEGORIA = c_idcategoria;

    EX_INS_NUL          EXCEPTION;  -- no es poden insertar valors nulls si estan definits com NOT NULL
    EX_FK_NUL           EXCEPTION;  -- no existeix la clau forana
    EX_NCAT_NOK         EXCEPTION;  -- la categoria no existeix 
    EX_PRO_NOK          EXCEPTION;  -- no hi han productes actius d'aquesta categoria
    
    PRAGMA EXCEPTION_INIT(EX_INS_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);


BEGIN

     v_prodediment :=   'CONSULTA 6 ';	
     v_param_entrada := 'CODI CATEGORIA: '||c_idcategoria;

-- VERIFIQUEM QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULL
        IF (c_idcategoria IS NULL ) THEN
            RAISE EX_INS_NUL;
        END IF; 
        
-- MIRAR SI LA CATEGORIA ES CORRECTA
        SELECT COUNT(1) INTO v_ntest
           FROM CATEGORIA
            WHERE IDCATEGORIA = c_idcategoria;
        IF (v_ntest=0) THEN
           RAISE EX_NCAT_NOK;
        END IF;          
        
-- OBTENIM EL NOM DE LA CATEGORIA     
        SELECT NOMCATEGORIA INTO v_nomcateg
           FROM CATEGORIA
            WHERE IDCATEGORIA = c_idcategoria;
                    
    -- MIRAR SI HI HAN PRODUCTES ACTIUS ASSOCIATS A AQUESTA CATEGORIA
        SELECT  COUNT(1) INTO  v_ntest
           FROM PRODUCTE
            WHERE IDAPLIC IN (SELECT IDAPLICACIO FROM APLICACIO WHERE IDCATEGORIA =c_idcategoria)
                          AND producte.estatversio = 'A';
        IF (v_ntest=0) THEN
           RAISE EX_PRO_NOK;
        END IF;

        DBMS_OUTPUT.PUT_LINE (' ');
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------');
        DBMS_OUTPUT.PUT_LINE ('Donada una determinada categoria, el llistat de les aplicacions actives per descarregar relacionades amb aquesta categoria');
        DBMS_OUTPUT.PUT_LINE ('ordenat per nom de: APLICACIO, SSOO, VERSIO, entenent com aplicacions: APLICACIO + SSOO + VERSIO ');
        DBMS_OUTPUT.PUT_LINE ('Codi Categoria :'||c_idcategoria||' /  Nom Categoria: '||v_nomcateg);
        DBMS_OUTPUT.PUT_LINE ('Data del llistat :'||to_char(sysdate,'Day, DD "de" MONTH "de" YYYY'));
        DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------');

        DBMS_OUTPUT.PUT_LINE (rpad('Data pujada',16)
                            ||' '||rpad('Codi ',5)
                            ||' '||rpad('Aplicacio ',20)
                            ||' '||rpad('SS OO',20)
                            ||' '||rpad('Versio',8)
                            ||' '||rpad('Mida',15)
                            ||' '||rpad('Resoluci',15));
         DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------');                   
        
      -- APERTURA DEL CURSOR
          OPEN C6;
      -- EXTRACCIO DE LES DADES
          LOOP
          FETCH C6 INTO
                         v_idproducte, v_datapujada , v_nomaplic, v_nomssoo, v_versio, 
                         v_midaapli, v_resoulucio, v_idcat;
                      
          EXIT WHEN C6%NOTFOUND;
          DBMS_OUTPUT.PUT_LINE (rpad(v_datapujada,16)
                            ||' '||rpad(v_idproducte,5)
                            ||' '||rpad(v_nomaplic,20)
                            ||' '||rpad(v_nomssoo,20)
                            ||' '||rpad(v_versio,8)
                            ||' '||rpad(v_midaapli,15)
                            ||' '||rpad(v_resoulucio,15));
          DBMS_OUTPUT.PUT_LINE (' ');
          END LOOP;
          DBMS_OUTPUT.PUT_LINE (rpad('Total aplicacions:',20)||' ' ||rpad(v_ntest,20));
       -- TANCAR EL CURSOR   
          CLOSE C6;

      -- FINALITZA AMB EXIT
          	c_rsp := 'OK'; 
            DBMS_OUTPUT.PUT_LINE(v_prodediment|| 'RSP: '||c_rsp);
            
       -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);        
      
      COMMIT;
        
       EXCEPTION 
            WHEN EX_INS_NUL THEN   -- CONTROL ATRIBUTS NOT NULL
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: EL PARAMETRE ENTRADA NO POT SER NULL '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

           WHEN EX_FK_NUL THEN   -- CONTROL QUE EXISTEIXIN LES CLAUS FORANES
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: CLAU FORANA NO EXISTENT '||v_error|| ': '||v_desc;

            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);

          WHEN EX_NCAT_NOK THEN   -- CONTROL: la categoria no existeix 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: la categoria no existeix   '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);     
         
          WHEN EX_PRO_NOK THEN   -- CONTROL: no hi han productes actius d'aquesta categoria 
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR: no hi han productes actius amb aquesta categoria  '||v_error|| ': '||v_desc;
             
            -- INSERTEM L'ERROR A LA TAULA DE LOGS
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);          

          WHEN OTHERS THEN           -- CONTROL D'ALTES ERRORS
               v_error := SQLCODE;
               v_desc := SUBSTR(SQLERRM, 1, 256);
               c_rsp := v_prodediment||' ERROR !!! '||v_error || ': '||v_desc ;
               
            -- INSERTEM L'ERROR A LA TAULA DE LOG
               ALTA_LOGS(v_prodediment, v_param_entrada ,'RSP: '||c_rsp);
               DBMS_OUTPUT.PUT_LINE('RSP: '||c_rsp);
END CONSULTA_6;

/
spool OFF